# This script does the same as action_count.py, using the attributes of opm_embedded instead of the run function

import opm_embedded

summary_state = opm_embedded.current_summary_state

if (not 'stop_on_next_run' in locals()): # Define this variable on the first run
    stop_on_next_run = False

if (not stop_on_next_run):
    if not "run_count" in summary_state:
        summary_state["run_count"] = 0
    summary_state["run_count"] += 1

if summary_state.elapsed() > (365 + 15)*24*3600:
    stop_on_next_run = True



import unittest
import numpy as np

from opm.io.ecl import EGrid
try:
    from tests.utils import test_path
except ImportError:
    from utils import test_path



class TestEGrid(unittest.TestCase):

    def test_ijk_active_and_global_indices(self):

        with self.assertRaises(RuntimeError):
            EGrid("/file/that/does_not_exists")

        grid1 = EGrid(test_path("data/9_EDITNNC.EGRID"))

        self.assertEqual(grid1.active_cells, 2794)

        nI, nJ, nK = grid1.dimension
        tot_ant_cells = nI * nJ * nK

        self.assertEqual(nI, 13)
        self.assertEqual(nJ, 22)
        self.assertEqual(nK, 11)

        i, j, k = grid1.ijk_from_global_index(0)

        self.assertEqual(i, 0)
        self.assertEqual(j, 0)
        self.assertEqual(k, 0)

        i, j, k = grid1.ijk_from_global_index(1000)

        self.assertEqual(i, 12)
        self.assertEqual(j, 10)
        self.assertEqual(k, 3)

        self.assertEqual(grid1.global_index(12, 10, 3), 1000)

        i, j, k = grid1.ijk_from_global_index(tot_ant_cells - 1)

        self.assertEqual(i, nI - 1)
        self.assertEqual(j, nJ - 1)
        self.assertEqual(k, nK - 1)

        with self.assertRaises(ValueError):
            i,j,k = grid1.ijk_from_global_index(tot_ant_cells)

        i,j,k = grid1.ijk_from_active_index(1000)

        self.assertEqual(i, 1)
        self.assertEqual(j, 15)
        self.assertEqual(k, 3)

        self.assertEqual( grid1.active_index(1, 15, 3), 1000)


    def test_mapaxes(self):

        axsref = [304.8, 152.4, 304.8, 457.2, 609.6, 457.2]
        X1ref = [304.8, 305.8, 304.8, 305.8, 304.8, 305.8, 304.8, 305.8]
        Y1ref = [457.2, 457.2, 456.2, 456.2, 457.2, 457.2, 456.2, 456.2]

        grid1 = EGrid(test_path("data/MAPAXES.EGRID"))

        X1, Y1, _ = grid1.xyz_from_ijk(0, 0, 0, True)

        for n in range(0, 8):
            self.assertAlmostEqual(X1ref[n], X1[n], 8)
            self.assertAlmostEqual(Y1ref[n], Y1[n], 8)

        axs = grid1.export_mapaxes()

        for n in range(0, 6):
            self.assertAlmostEqual(axsref[n], axs[n], 8)

        X2ref = [0., 1., 0., 1., 0., 1., 0., 1.]
        Y2ref = [0., 0., 1., 1., 0., 0., 1., 1.]

        X2, Y2, _ = grid1.xyz_from_ijk(0, 0, 0)

        for n in range(0, 8):
            self.assertAlmostEqual(X2ref[n], X2[n], 8)
            self.assertAlmostEqual(Y2ref[n], Y2[n], 8)


    def test_coordinates(self):

        Xref = [2899.45166015625, 2999.390869140625, 2899.45166015625, 2999.390869140625,
               2899.4176237656716, 2999.3568089317187, 2899.417623015281, 2999.356808099622]
        Yref = [2699.973388671875, 2699.973388671875, 2799.969482421875, 2799.969482421875,
               2699.9818918149376, 2699.9818918149376, 2799.978009571257, 2799.9780095915808]
        Zref = [2565.301025390625, 2568.791015625, 2564.42822265625, 2567.918212890625,
               2575.29443359375, 2578.784423828125, 2574.421875, 2577.911865234375]

        grid1 = EGrid(test_path("data/9_EDITNNC.EGRID"))

        X1, Y1, Z1 = grid1.xyz_from_ijk(9, 7, 0)

        for n in range(0, 8):
            self.assertAlmostEqual(Xref[n], X1[n], 8)
            self.assertAlmostEqual(Yref[n], Y1[n], 8)
            self.assertAlmostEqual(Zref[n], Z1[n], 8)

        actInd = grid1.active_index(9, 7, 0)
        X2, Y2, Z2 = grid1.xyz_from_active_index(actInd)

        for n in range(0, 8):
            self.assertAlmostEqual(Xref[n], X2[n], 8)
            self.assertAlmostEqual(Yref[n], Y2[n], 8)
            self.assertAlmostEqual(Zref[n], Z2[n], 8)


    def test_cell_volume(self):

        grid1 = EGrid(test_path("data/9_EDITNNC.EGRID"))

        nI, nJ, nK = grid1.dimension
        tot_ant_cells = nI * nJ * nK

        celVolAll = grid1.cellvolumes()

        self.assertTrue(isinstance(celVolAll, np.ndarray))
        self.assertEqual(len(celVolAll), tot_ant_cells)

        self.assertTrue(min(celVolAll) > 0.0)

        mask =[0] * tot_ant_cells

        for k in range(nK):
            globInd = grid1.global_index(0, 0, k)
            mask[globInd] = 1

        celVol = grid1.cellvolumes(mask)

        self.assertTrue(min(celVol) == 0.0)
        self.assertEqual(np.count_nonzero(celVol), nK)


if __name__ == "__main__":

    unittest.main()



import unittest
import sys
import numpy as np
import io

from opm.io.ecl import ERst, eclArrType, EclOutput

try:
    from tests.utils import test_path
except ImportError:
    from utils import test_path


class TestERst(unittest.TestCase):

    def test_reportSteps(self):

        rst1 = ERst(test_path("data/SPE9.UNRST"))

        self.assertTrue( 37 in rst1)
        self.assertTrue( 74 in rst1)

        self.assertRaises(ValueError, rst1.load_report_step, 137 );

        rst1.load_report_step(37);
        rst1.load_report_step(74);

        self.assertTrue(37 in rst1.report_steps)
        self.assertTrue(74 in rst1.report_steps)

        self.assertEqual(len(rst1), 2)


    def test_contains(self):

        rst1 = ERst(test_path("data/SPE9.UNRST"))

        self.assertTrue(("PRESSURE", 37) in rst1)
        self.assertTrue(("SWAT", 74) in rst1)

        arrayList37=rst1.arrays(37)

        self.assertEqual(len(arrayList37), 21)

        arrName, arrType, arrSize = arrayList37[16]

        self.assertEqual(arrName, "PRESSURE")
        self.assertEqual(arrType,  eclArrType.REAL)
        self.assertEqual(arrSize, 9000)


    def test_getitem(self):

        rst1 = ERst(test_path("data/SPE9.UNRST"))

        # get first occurrence of ZWEL, report step 37
        zwel1 = rst1[11, 37]

        zwel2 = rst1["ZWEL",37, 0]
        zwel3 = rst1["ZWEL",37]

        for v1,v2 in zip (zwel1, zwel2):
            self.assertEqual(v1, v2)

        for v1,v2 in zip (zwel1, zwel3):
            self.assertEqual(v1, v2)

        self.assertEqual(len(zwel1), 78)

        self.assertEqual(zwel1[0], "INJE1")
        self.assertEqual(zwel1[3], "PRODU2")
        self.assertEqual(zwel1[6], "PRODU3")

        # get first occurrence of INTEHEAD, report step 37
        inteh = rst1["INTEHEAD",37]

        self.assertEqual(len(inteh), 411)
        self.assertTrue(isinstance(inteh, np.ndarray))
        self.assertEqual(inteh.dtype, "int32")

        self.assertEqual(inteh[1], 201702)
        self.assertEqual(inteh[9], 25)
        self.assertEqual(inteh[64], 6)
        self.assertEqual(inteh[65], 1)
        self.assertEqual(inteh[66], 2016)

        # get first occurrence of PRESSURE, report step 74
        pres74 = rst1["PRESSURE",74]

        self.assertTrue(isinstance(pres74, np.ndarray))
        self.assertEqual(pres74.dtype, "float32")
        self.assertEqual(len(pres74), 9000)

        self.assertAlmostEqual(pres74[0], 2290.9192, 4)
        self.assertAlmostEqual(pres74[1], 2254.6619, 4)
        self.assertAlmostEqual(pres74[2], 2165.5347, 4)
        self.assertAlmostEqual(pres74[3], 1996.2598, 4)

        xcon = rst1["XCON", 74]
        self.assertTrue(isinstance(xcon, np.ndarray))
        self.assertEqual(xcon.dtype, "float64")
        self.assertEqual(len(xcon), 7540)

        self.assertAlmostEqual(xcon[1], -22.841887080742975, 10)

        logih = rst1["LOGIHEAD", 74]
        self.assertTrue(isinstance(logih, np.ndarray))
        self.assertEqual(len(logih), 121)

        for b1, b2 in zip([True, True, False, False, False], logih[0:5]):
            self.assertEqual(b1, b2)


    def test_getby_index(self):

        rst1 = ERst(test_path("data/SPE9.UNRST"))

        self.assertTrue(("SGAS", 74) in rst1)
        self.assertTrue(rst1.count("SGAS", 74), 1)

        arrayList74=rst1.arrays(74)

        array_name_list = [item[0] for item in arrayList74]
        ind =  array_name_list.index("SGAS")

        sgas_a = rst1[ind, 74]
        sgas_b = rst1["SGAS", 74]

        self.assertEqual(len(sgas_a), len(sgas_b))

        for sg1, sg2 in zip(sgas_a, sgas_b):
            self.assertEqual(sg1, sg2)


    def test_list_of_arrays(self):

        refArrList = ["SEQNUM", "INTEHEAD", "LOGIHEAD", "DOUBHEAD", "IGRP", "SGRP", "XGRP", "ZGRP", "IWEL",
                      "SWEL","XWEL","ZWEL", "ICON", "SCON", "XCON", "STARTSOL","PRESSURE", "RS", "SGAS", "SWAT", "ENDSOL"]

        rst1 = ERst(test_path("data/SPE9.UNRST"))

        array_list_74=rst1.arrays(74)

        self.assertEqual(len(refArrList), len(array_list_74) )

        for n, (name, arrType, arrSize) in enumerate(array_list_74):

            self.assertEqual(name, refArrList[n])

            if arrType != eclArrType.MESS:
                array = rst1[name, 74]
                self.assertEqual(len(array), arrSize)

            if arrType == eclArrType.INTE:
                self.assertEqual(array.dtype, "int32")
            elif arrType == eclArrType.REAL:
                self.assertEqual(array.dtype, "float32")
            elif arrType == eclArrType.DOUB:
                self.assertEqual(array.dtype, "float64")
            elif arrType == eclArrType.LOGI:
                self.assertEqual(array.dtype, "bool")
            elif arrType == eclArrType.CHAR:
                self.assertTrue(isinstance(array, list))


    def test_get_occurence(self):

        testFile = test_path("data/TMP.UNRST")

        npArr11 = np.array([11.1, 11.2, 11.3, 11.4], dtype='float32')
        npArr12 = np.array([12.1, 12.2, 12.3, 12.4], dtype='float32')
        npArr13 = np.array([13.1, 13.2, 13.3, 13.4], dtype='float32')

        npArr21 = np.array([21.1, 21.2, 21.3, 21.4], dtype='float32')
        npArr22 = np.array([22.1, 22.2, 22.3, 22.4], dtype='float32')
        npArr23 = np.array([23.1, 23.2, 23.3, 23.4], dtype='float32')

        seqn1 = np.array([1], dtype='int32')
        seqn2 = np.array([2], dtype='int32')

        out1 = EclOutput(testFile)

        out1.write("SEQNUM", seqn1)
        out1.write("TESTING", npArr11)
        out1.write("TESTING", npArr12)
        out1.write("TESTING", npArr13)

        out1.write("SEQNUM", seqn2)
        out1.write("TESTING", npArr21)
        out1.write("TESTING", npArr22)
        out1.write("TESTING", npArr23)

        rst1 = ERst(testFile)

        with self.assertRaises(IndexError):
            arr = rst1["TESTING", 1, 3]
            arr = rst1["TESTING", 2, 3]

        test_npArr11 = rst1["TESTING",1, 0]
        test_npArr12 = rst1["TESTING",1, 1]
        test_npArr13 = rst1["TESTING",1, 2]

        self.assertTrue( np.array_equal(test_npArr11, npArr11) )
        self.assertTrue( np.array_equal(test_npArr12, npArr12) )
        self.assertTrue( np.array_equal(test_npArr13, npArr13) )

        test_npArr21 = rst1["TESTING",2, 0]
        test_npArr22 = rst1["TESTING",2, 1]
        test_npArr23 = rst1["TESTING",2, 2]

        self.assertTrue( np.array_equal(test_npArr21, npArr21) )
        self.assertTrue( np.array_equal(test_npArr22, npArr22) )
        self.assertTrue( np.array_equal(test_npArr23, npArr23) )


if __name__ == "__main__":

    unittest.main()



from opm._common import EModel, eclArrType, EModel_add_filter_docstring
from opm._common import calc_cell_vol

def emodel_add_filter(self, key, operator, val1, val2 = None):

    if key in ["I", "J", "K", "ROW", "COLUMN", "LAYER"]:
        arrType=eclArrType.INTE
    else:
        arrList = self.get_list_of_arrays()
        arrNameList = [item[0] for item in arrList]
        arrTypeList = [item[1] for item in arrList]

        if key not in arrNameList:
            raise ValueError("{} array not found in EModel".format(key))

        ind = arrNameList.index(key)

        arrType = arrTypeList[ind]

    if not val2:
        if arrType == eclArrType.INTE:
            self.__add_filter(key, operator, int(val1))
        elif arrType == eclArrType.REAL:
            self.__add_filter(key, operator, float(val1))
    else:
        if arrType == eclArrType.INTE:
            self.__add_filter(key, operator, int(val1), int(val2))
        elif arrType == eclArrType.REAL:
            self.__add_filter(key, operator, float(val1), float(val2))

emodel_add_filter.__doc__ = EModel_add_filter_docstring

setattr(EModel, "add_filter", emodel_add_filter)



def run(ecl_state, schedule, report_step, summary_state, actionx_callback):
	if (report_step == 2):
		wconprod_shut = """
		WCONPROD
			'{}' '{}' '{}' {} 4* {} /
		/
			""".format('P1', 'SHUT', 'ORAT', 4000, 1000)
		schedule.insert_keywords(wconprod_shut)
		wconprod_open = """
		WCONPROD
			'{}' '{}' '{}' {} 4* {} /
		/
			""".format('P1', 'OPEN', 'ORAT', 4000, 1000)
		schedule.insert_keywords(wconprod_open, report_step + 1)



from math import sin
import random

def run():
    pass

print("sin(0) = {}".format(sin(0)))
#---
if random.random() > 0.25:
    print("Large outcome")
else:
    print("Small result")
A = 100
B = A / 10
C = B * 20



from typing import Any, ClassVar, Dict, Iterator, List, Tuple

from typing import overload
import datetime
import numpy
C0nn: eclArrType
CHAR: eclArrType
DOUB: eclArrType
EDIT: eclSectionType
EModel_add_filter_docstring: str
GRID: eclSectionType
INTE: eclArrType
LOGI: eclArrType
MESS: eclArrType
PROPS: eclSectionType
REAL: eclArrType
REGIONS: eclSectionType
RUNSPEC: eclSectionType
SCHEDULE: eclSectionType
SOLUTION: eclSectionType
SUMMARY: eclSectionType

current_ecl_state: EclipseState
current_report_step: int
current_schedule: Schedule
current_summary_state: SummaryState

class Builtin:
    def __init__(self) -> None: ...
    def __getitem__(self, index) -> Any: ...
    @property
    def ACF(self) -> Any: ...
    @property
    def ACTCO2S(self) -> Any: ...
    @property
    def ACTDIMS(self) -> Any: ...
    @property
    def ACTION(self) -> Any: ...
    @property
    def ACTIONG(self) -> Any: ...
    @property
    def ACTIONR(self) -> Any: ...
    @property
    def ACTIONS(self) -> Any: ...
    @property
    def ACTIONW(self) -> Any: ...
    @property
    def ACTIONX(self) -> Any: ...
    @property
    def ACTNUM(self) -> Any: ...
    @property
    def ACTPARAM(self) -> Any: ...
    @property
    def ADD(self) -> Any: ...
    @property
    def ADDREG(self) -> Any: ...
    @property
    def ADDZCORN(self) -> Any: ...
    @property
    def ADSALNOD(self) -> Any: ...
    @property
    def ADSORP(self) -> Any: ...
    @property
    def AITS(self) -> Any: ...
    @property
    def AITSOFF(self) -> Any: ...
    @property
    def ALKADS(self) -> Any: ...
    @property
    def ALKALINE(self) -> Any: ...
    @property
    def ALKROCK(self) -> Any: ...
    @property
    def ALL(self) -> Any: ...
    @property
    def ALPOLADS(self) -> Any: ...
    @property
    def ALSURFAD(self) -> Any: ...
    @property
    def ALSURFST(self) -> Any: ...
    @property
    def AMALGAM(self) -> Any: ...
    @property
    def API(self) -> Any: ...
    @property
    def APIGROUP(self) -> Any: ...
    @property
    def APILIM(self) -> Any: ...
    @property
    def APIVD(self) -> Any: ...
    @property
    def AQANCONL(self) -> Any: ...
    @property
    def AQANNC(self) -> Any: ...
    @property
    def AQANTRC(self) -> Any: ...
    @property
    def AQUALIST(self) -> Any: ...
    @property
    def AQUANCON(self) -> Any: ...
    @property
    def AQUCHGAS(self) -> Any: ...
    @property
    def AQUCHWAT(self) -> Any: ...
    @property
    def AQUCON(self) -> Any: ...
    @property
    def AQUCT(self) -> Any: ...
    @property
    def AQUCWFAC(self) -> Any: ...
    @property
    def AQUDIMS(self) -> Any: ...
    @property
    def AQUFET(self) -> Any: ...
    @property
    def AQUFETP(self) -> Any: ...
    @property
    def AQUFLUX(self) -> Any: ...
    @property
    def AQUIFER_PROBE_ANALYTIC(self) -> Any: ...
    @property
    def AQUIFER_PROBE_ANALYTIC_NAMED(self) -> Any: ...
    @property
    def AQUIFER_PROBE_NUMERIC(self) -> Any: ...
    @property
    def AQUNNC(self) -> Any: ...
    @property
    def AQUNUM(self) -> Any: ...
    @property
    def AQUTAB(self) -> Any: ...
    @property
    def AUTOCOAR(self) -> Any: ...
    @property
    def AUTOREF(self) -> Any: ...
    @property
    def BC(self) -> Any: ...
    @property
    def BCCON(self) -> Any: ...
    @property
    def BCPROP(self) -> Any: ...
    @property
    def BDENSITY(self) -> Any: ...
    @property
    def BGGI(self) -> Any: ...
    @property
    def BIC(self) -> Any: ...
    @property
    def BIGMODEL(self) -> Any: ...
    @property
    def BIOFILM(self) -> Any: ...
    @property
    def BIOFPARA(self) -> Any: ...
    @property
    def BIOTCOEF(self) -> Any: ...
    @property
    def BLACKOIL(self) -> Any: ...
    @property
    def BLOCK_PROBE(self) -> Any: ...
    @property
    def BLOCK_PROBE300(self) -> Any: ...
    @property
    def BLOCK_PROBE900(self) -> Any: ...
    @property
    def BOGI(self) -> Any: ...
    @property
    def BOUNDARY(self) -> Any: ...
    @property
    def BOX(self) -> Any: ...
    @property
    def BPARA(self) -> Any: ...
    @property
    def BPIDIMS(self) -> Any: ...
    @property
    def BRANPROP(self) -> Any: ...
    @property
    def BRINE(self) -> Any: ...
    @property
    def BTOBALFA(self) -> Any: ...
    @property
    def BTOBALFV(self) -> Any: ...
    @property
    def CALTRAC(self) -> Any: ...
    @property
    def CARFIN(self) -> Any: ...
    @property
    def CART(self) -> Any: ...
    @property
    def CBMOPTS(self) -> Any: ...
    @property
    def CECON(self) -> Any: ...
    @property
    def CECONT(self) -> Any: ...
    @property
    def CIRCLE(self) -> Any: ...
    @property
    def CNAMES(self) -> Any: ...
    @property
    def CO2SOL(self) -> Any: ...
    @property
    def CO2STOR(self) -> Any: ...
    @property
    def CO2STORE(self) -> Any: ...
    @property
    def COAL(self) -> Any: ...
    @property
    def COALADS(self) -> Any: ...
    @property
    def COALNUM(self) -> Any: ...
    @property
    def COALPP(self) -> Any: ...
    @property
    def COARSEN(self) -> Any: ...
    @property
    def COLLAPSE(self) -> Any: ...
    @property
    def COLUMNS(self) -> Any: ...
    @property
    def COMPDAT(self) -> Any: ...
    @property
    def COMPDATX(self) -> Any: ...
    @property
    def COMPFLSH(self) -> Any: ...
    @property
    def COMPIMB(self) -> Any: ...
    @property
    def COMPINJK(self) -> Any: ...
    @property
    def COMPLMPL(self) -> Any: ...
    @property
    def COMPLUMP(self) -> Any: ...
    @property
    def COMPOFF(self) -> Any: ...
    @property
    def COMPORD(self) -> Any: ...
    @property
    def COMPRIV(self) -> Any: ...
    @property
    def COMPRP(self) -> Any: ...
    @property
    def COMPRPL(self) -> Any: ...
    @property
    def COMPS(self) -> Any: ...
    @property
    def COMPSEGL(self) -> Any: ...
    @property
    def COMPSEGS(self) -> Any: ...
    @property
    def COMPTRAJ(self) -> Any: ...
    @property
    def COMPVE(self) -> Any: ...
    @property
    def COMPVEL(self) -> Any: ...
    @property
    def CONNECTION_PROBE(self) -> Any: ...
    @property
    def CONNECTION_PROBE_OPM(self) -> Any: ...
    @property
    def COORD(self) -> Any: ...
    @property
    def COORDSYS(self) -> Any: ...
    @property
    def COPY(self) -> Any: ...
    @property
    def COPYBOX(self) -> Any: ...
    @property
    def COPYREG(self) -> Any: ...
    @property
    def CPIFACT(self) -> Any: ...
    @property
    def CPIFACTL(self) -> Any: ...
    @property
    def CPR(self) -> Any: ...
    @property
    def CREF(self) -> Any: ...
    @property
    def CREFW(self) -> Any: ...
    @property
    def CREFWS(self) -> Any: ...
    @property
    def CRITPERM(self) -> Any: ...
    @property
    def CSKIN(self) -> Any: ...
    @property
    def DATE(self) -> Any: ...
    @property
    def DATES(self) -> Any: ...
    @property
    def DATUM(self) -> Any: ...
    @property
    def DATUMR(self) -> Any: ...
    @property
    def DATUMRX(self) -> Any: ...
    @property
    def DCQDEFN(self) -> Any: ...
    @property
    def DEBUG_(self) -> Any: ...
    @property
    def DELAYACT(self) -> Any: ...
    @property
    def DENAQA(self) -> Any: ...
    @property
    def DENSITY(self) -> Any: ...
    @property
    def DEPTH(self) -> Any: ...
    @property
    def DEPTHTAB(self) -> Any: ...
    @property
    def DEPTHZ(self) -> Any: ...
    @property
    def DIAGDISP(self) -> Any: ...
    @property
    def DIFF(self) -> Any: ...
    @property
    def DIFFAGAS(self) -> Any: ...
    @property
    def DIFFAWAT(self) -> Any: ...
    @property
    def DIFFC(self) -> Any: ...
    @property
    def DIFFCGAS(self) -> Any: ...
    @property
    def DIFFCOAL(self) -> Any: ...
    @property
    def DIFFCWAT(self) -> Any: ...
    @property
    def DIFFDP(self) -> Any: ...
    @property
    def DIFFMICP(self) -> Any: ...
    @property
    def DIFFMMF(self) -> Any: ...
    @property
    def DIFFMR(self) -> Any: ...
    @property
    def DIFFMTHT(self) -> Any: ...
    @property
    def DIFFMX(self) -> Any: ...
    @property
    def DIFFMY(self) -> Any: ...
    @property
    def DIFFMZ(self) -> Any: ...
    @property
    def DIFFR(self) -> Any: ...
    @property
    def DIFFTHT(self) -> Any: ...
    @property
    def DIFFUSE(self) -> Any: ...
    @property
    def DIFFX(self) -> Any: ...
    @property
    def DIFFY(self) -> Any: ...
    @property
    def DIFFZ(self) -> Any: ...
    @property
    def DIMENS(self) -> Any: ...
    @property
    def DIMPES(self) -> Any: ...
    @property
    def DIMPLICT(self) -> Any: ...
    @property
    def DISGAS(self) -> Any: ...
    @property
    def DISGASW(self) -> Any: ...
    @property
    def DISPDIMS(self) -> Any: ...
    @property
    def DISPERC(self) -> Any: ...
    @property
    def DISPERSE(self) -> Any: ...
    @property
    def DOMAINS(self) -> Any: ...
    @property
    def DPGRID(self) -> Any: ...
    @property
    def DPKRMOD(self) -> Any: ...
    @property
    def DPNUM(self) -> Any: ...
    @property
    def DR(self) -> Any: ...
    @property
    def DREF(self) -> Any: ...
    @property
    def DREFS(self) -> Any: ...
    @property
    def DRILPRI(self) -> Any: ...
    @property
    def DRSDT(self) -> Any: ...
    @property
    def DRSDTCON(self) -> Any: ...
    @property
    def DRSDTR(self) -> Any: ...
    @property
    def DRV(self) -> Any: ...
    @property
    def DRVDT(self) -> Any: ...
    @property
    def DRVDTR(self) -> Any: ...
    @property
    def DSPDEINT(self) -> Any: ...
    @property
    def DTHETA(self) -> Any: ...
    @property
    def DTHETAV(self) -> Any: ...
    @property
    def DUALPERM(self) -> Any: ...
    @property
    def DUALPORO(self) -> Any: ...
    @property
    def DUMPCUPL(self) -> Any: ...
    @property
    def DUMPFLUX(self) -> Any: ...
    @property
    def DX(self) -> Any: ...
    @property
    def DXV(self) -> Any: ...
    @property
    def DY(self) -> Any: ...
    @property
    def DYNAMICR(self) -> Any: ...
    @property
    def DYNRDIMS(self) -> Any: ...
    @property
    def DYV(self) -> Any: ...
    @property
    def DZ(self) -> Any: ...
    @property
    def DZMATRIX(self) -> Any: ...
    @property
    def DZMTRX(self) -> Any: ...
    @property
    def DZMTRXV(self) -> Any: ...
    @property
    def DZNET(self) -> Any: ...
    @property
    def DZV(self) -> Any: ...
    @property
    def ECHO(self) -> Any: ...
    @property
    def ECLMC(self) -> Any: ...
    @property
    def EDIT(self) -> Any: ...
    @property
    def EDITNNC(self) -> Any: ...
    @property
    def EDITNNCR(self) -> Any: ...
    @property
    def EHYSTR(self) -> Any: ...
    @property
    def EHYSTRR(self) -> Any: ...
    @property
    def END(self) -> Any: ...
    @property
    def ENDACTIO(self) -> Any: ...
    @property
    def ENDBOX(self) -> Any: ...
    @property
    def ENDDYN(self) -> Any: ...
    @property
    def ENDFIN(self) -> Any: ...
    @property
    def ENDINC(self) -> Any: ...
    @property
    def ENDNUM(self) -> Any: ...
    @property
    def ENDPOINT_SPECIFIERS(self) -> Any: ...
    @property
    def ENDSCALE(self) -> Any: ...
    @property
    def ENDSKIP(self) -> Any: ...
    @property
    def ENKRVD(self) -> Any: ...
    @property
    def ENPCVD(self) -> Any: ...
    @property
    def ENPTVD(self) -> Any: ...
    @property
    def ENSPCVD(self) -> Any: ...
    @property
    def EOS(self) -> Any: ...
    @property
    def EOSNUM(self) -> Any: ...
    @property
    def EPSDBGS(self) -> Any: ...
    @property
    def EPSDEBUG(self) -> Any: ...
    @property
    def EQLDIMS(self) -> Any: ...
    @property
    def EQLNUM(self) -> Any: ...
    @property
    def EQLOPTS(self) -> Any: ...
    @property
    def EQLZCORN(self) -> Any: ...
    @property
    def EQUALREG(self) -> Any: ...
    @property
    def EQUALS(self) -> Any: ...
    @property
    def EQUIL(self) -> Any: ...
    @property
    def ESSNODE(self) -> Any: ...
    @property
    def EXCAVATE(self) -> Any: ...
    @property
    def EXCEL(self) -> Any: ...
    @property
    def EXIT(self) -> Any: ...
    @property
    def EXTFIN(self) -> Any: ...
    @property
    def EXTHOST(self) -> Any: ...
    @property
    def EXTRAPMS(self) -> Any: ...
    @property
    def EXTREPGL(self) -> Any: ...
    @property
    def FAULTDIM(self) -> Any: ...
    @property
    def FAULTS(self) -> Any: ...
    @property
    def FBHPDEF(self) -> Any: ...
    @property
    def FHERCHBL(self) -> Any: ...
    @property
    def FIELD(self) -> Any: ...
    @property
    def FIELDSEP(self) -> Any: ...
    @property
    def FIELD_PROBE(self) -> Any: ...
    @property
    def FIELD_PROBE_OPM(self) -> Any: ...
    @property
    def FILEUNIT(self) -> Any: ...
    @property
    def FILLEPS(self) -> Any: ...
    @property
    def FIPNUM(self) -> Any: ...
    @property
    def FIPOWG(self) -> Any: ...
    @property
    def FIPSEP(self) -> Any: ...
    @property
    def FIP_PROBE(self) -> Any: ...
    @property
    def FLUXNUM(self) -> Any: ...
    @property
    def FLUXREG(self) -> Any: ...
    @property
    def FLUXTYPE(self) -> Any: ...
    @property
    def FMTHMD(self) -> Any: ...
    @property
    def FMTIN(self) -> Any: ...
    @property
    def FMTOUT(self) -> Any: ...
    @property
    def FMWSET(self) -> Any: ...
    @property
    def FOAM(self) -> Any: ...
    @property
    def FOAMADS(self) -> Any: ...
    @property
    def FOAMDCYO(self) -> Any: ...
    @property
    def FOAMDCYW(self) -> Any: ...
    @property
    def FOAMFCN(self) -> Any: ...
    @property
    def FOAMFRM(self) -> Any: ...
    @property
    def FOAMFSC(self) -> Any: ...
    @property
    def FOAMFSO(self) -> Any: ...
    @property
    def FOAMFST(self) -> Any: ...
    @property
    def FOAMFSW(self) -> Any: ...
    @property
    def FOAMMOB(self) -> Any: ...
    @property
    def FOAMMOBP(self) -> Any: ...
    @property
    def FOAMMOBS(self) -> Any: ...
    @property
    def FOAMOPTS(self) -> Any: ...
    @property
    def FOAMROCK(self) -> Any: ...
    @property
    def FORMFEED(self) -> Any: ...
    @property
    def FRICTION(self) -> Any: ...
    @property
    def FULLIMP(self) -> Any: ...
    @property
    def GAS(self) -> Any: ...
    @property
    def GASBEGIN(self) -> Any: ...
    @property
    def GASCONC(self) -> Any: ...
    @property
    def GASDENT(self) -> Any: ...
    @property
    def GASEND(self) -> Any: ...
    @property
    def GASFCOMP(self) -> Any: ...
    @property
    def GASFDECR(self) -> Any: ...
    @property
    def GASFDELC(self) -> Any: ...
    @property
    def GASFIELD(self) -> Any: ...
    @property
    def GASFTARG(self) -> Any: ...
    @property
    def GASJT(self) -> Any: ...
    @property
    def GASMONTH(self) -> Any: ...
    @property
    def GASPERIO(self) -> Any: ...
    @property
    def GASSATC(self) -> Any: ...
    @property
    def GASVISCT(self) -> Any: ...
    @property
    def GASWAT(self) -> Any: ...
    @property
    def GASYEAR(self) -> Any: ...
    @property
    def GCALECON(self) -> Any: ...
    @property
    def GCOMPIDX(self) -> Any: ...
    @property
    def GCONCAL(self) -> Any: ...
    @property
    def GCONENG(self) -> Any: ...
    @property
    def GCONINJE(self) -> Any: ...
    @property
    def GCONPRI(self) -> Any: ...
    @property
    def GCONPROD(self) -> Any: ...
    @property
    def GCONSALE(self) -> Any: ...
    @property
    def GCONSUMP(self) -> Any: ...
    @property
    def GCONTOL(self) -> Any: ...
    @property
    def GCUTBACK(self) -> Any: ...
    @property
    def GCUTBACT(self) -> Any: ...
    @property
    def GCVD(self) -> Any: ...
    @property
    def GDCQ(self) -> Any: ...
    @property
    def GDCQECON(self) -> Any: ...
    @property
    def GDFILE(self) -> Any: ...
    @property
    def GDIMS(self) -> Any: ...
    @property
    def GDORIENT(self) -> Any: ...
    @property
    def GDRILPOT(self) -> Any: ...
    @property
    def GECON(self) -> Any: ...
    @property
    def GECONT(self) -> Any: ...
    @property
    def GEFAC(self) -> Any: ...
    @property
    def GETDATA(self) -> Any: ...
    @property
    def GETGLOB(self) -> Any: ...
    @property
    def GI(self) -> Any: ...
    @property
    def GIALL(self) -> Any: ...
    @property
    def GIMODEL(self) -> Any: ...
    @property
    def GINODE(self) -> Any: ...
    @property
    def GLIFTLIM(self) -> Any: ...
    @property
    def GLIFTOPT(self) -> Any: ...
    @property
    def GMWSET(self) -> Any: ...
    @property
    def GNETDP(self) -> Any: ...
    @property
    def GNETINJE(self) -> Any: ...
    @property
    def GNETPUMP(self) -> Any: ...
    @property
    def GPMAINT(self) -> Any: ...
    @property
    def GRADGRUP(self) -> Any: ...
    @property
    def GRADRESV(self) -> Any: ...
    @property
    def GRADRFT(self) -> Any: ...
    @property
    def GRADWELL(self) -> Any: ...
    @property
    def GRAVCONS(self) -> Any: ...
    @property
    def GRAVDR(self) -> Any: ...
    @property
    def GRAVDRB(self) -> Any: ...
    @property
    def GRAVDRM(self) -> Any: ...
    @property
    def GRAVITY(self) -> Any: ...
    @property
    def GRDREACH(self) -> Any: ...
    @property
    def GRID(self) -> Any: ...
    @property
    def GRIDFILE(self) -> Any: ...
    @property
    def GRIDOPTS(self) -> Any: ...
    @property
    def GRIDUNIT(self) -> Any: ...
    @property
    def GROUP_PROBE(self) -> Any: ...
    @property
    def GROUP_PROBE_OPM(self) -> Any: ...
    @property
    def GRUPMAST(self) -> Any: ...
    @property
    def GRUPNET(self) -> Any: ...
    @property
    def GRUPRIG(self) -> Any: ...
    @property
    def GRUPSLAV(self) -> Any: ...
    @property
    def GRUPTARG(self) -> Any: ...
    @property
    def GRUPTREE(self) -> Any: ...
    @property
    def GSATINJE(self) -> Any: ...
    @property
    def GSATPROD(self) -> Any: ...
    @property
    def GSEPCOND(self) -> Any: ...
    @property
    def GSF(self) -> Any: ...
    @property
    def GSSCPTST(self) -> Any: ...
    @property
    def GSWINGF(self) -> Any: ...
    @property
    def GTADD(self) -> Any: ...
    @property
    def GTMULT(self) -> Any: ...
    @property
    def GUIDECAL(self) -> Any: ...
    @property
    def GUIDERAT(self) -> Any: ...
    @property
    def GUPFREQ(self) -> Any: ...
    @property
    def GWRTWCV(self) -> Any: ...
    @property
    def H2SOL(self) -> Any: ...
    @property
    def H2STORE(self) -> Any: ...
    @property
    def HALFTRAN(self) -> Any: ...
    @property
    def HAxxxxxx(self) -> Any: ...
    @property
    def HBNUM(self) -> Any: ...
    @property
    def HDISP(self) -> Any: ...
    @property
    def HEATCR(self) -> Any: ...
    @property
    def HEATCRT(self) -> Any: ...
    @property
    def HMAQUCT(self) -> Any: ...
    @property
    def HMAQUFET(self) -> Any: ...
    @property
    def HMAQUNUM(self) -> Any: ...
    @property
    def HMDIMS(self) -> Any: ...
    @property
    def HMFAULTS(self) -> Any: ...
    @property
    def HMMLAQUN(self) -> Any: ...
    @property
    def HMMLCTAQ(self) -> Any: ...
    @property
    def HMMLFTAQ(self) -> Any: ...
    @property
    def HMMLTWCN(self) -> Any: ...
    @property
    def HMMULTFT(self) -> Any: ...
    @property
    def HMMULTSG(self) -> Any: ...
    @property
    def HMMULTxx(self) -> Any: ...
    @property
    def HMPROPS(self) -> Any: ...
    @property
    def HMROCK(self) -> Any: ...
    @property
    def HMROCKT(self) -> Any: ...
    @property
    def HMRREF(self) -> Any: ...
    @property
    def HMWELCON(self) -> Any: ...
    @property
    def HMWPIMLT(self) -> Any: ...
    @property
    def HMxxxxxx(self) -> Any: ...
    @property
    def HRFIN(self) -> Any: ...
    @property
    def HWELLS(self) -> Any: ...
    @property
    def HWKRO(self) -> Any: ...
    @property
    def HWKRORG(self) -> Any: ...
    @property
    def HWKRORW(self) -> Any: ...
    @property
    def HWKRW(self) -> Any: ...
    @property
    def HWKRWR(self) -> Any: ...
    @property
    def HWPCW(self) -> Any: ...
    @property
    def HWSNUM(self) -> Any: ...
    @property
    def HWSOGCR(self) -> Any: ...
    @property
    def HWSOWCR(self) -> Any: ...
    @property
    def HWSWCR(self) -> Any: ...
    @property
    def HWSWL(self) -> Any: ...
    @property
    def HWSWLPC(self) -> Any: ...
    @property
    def HWSWU(self) -> Any: ...
    @property
    def HXFIN(self) -> Any: ...
    @property
    def HYDRHEAD(self) -> Any: ...
    @property
    def HYFIN(self) -> Any: ...
    @property
    def HYMOBGDR(self) -> Any: ...
    @property
    def HYST(self) -> Any: ...
    @property
    def HYSTCHCK(self) -> Any: ...
    @property
    def HZFIN(self) -> Any: ...
    @property
    def IHOST(self) -> Any: ...
    @property
    def IMBNUM(self) -> Any: ...
    @property
    def IMBNUMMF(self) -> Any: ...
    @property
    def IMKRVD(self) -> Any: ...
    @property
    def IMPCVD(self) -> Any: ...
    @property
    def IMPES(self) -> Any: ...
    @property
    def IMPLICIT(self) -> Any: ...
    @property
    def IMPORT(self) -> Any: ...
    @property
    def IMPTVD(self) -> Any: ...
    @property
    def IMSPCVD(self) -> Any: ...
    @property
    def INCLUDE(self) -> Any: ...
    @property
    def INIT(self) -> Any: ...
    @property
    def INRAD(self) -> Any: ...
    @property
    def INSPEC(self) -> Any: ...
    @property
    def INTPC(self) -> Any: ...
    @property
    def IONROCK(self) -> Any: ...
    @property
    def IONXROCK(self) -> Any: ...
    @property
    def IONXSURF(self) -> Any: ...
    @property
    def IPCG(self) -> Any: ...
    @property
    def IPCW(self) -> Any: ...
    @property
    def ISGCR(self) -> Any: ...
    @property
    def ISGL(self) -> Any: ...
    @property
    def ISGLPC(self) -> Any: ...
    @property
    def ISGU(self) -> Any: ...
    @property
    def ISOGCR(self) -> Any: ...
    @property
    def ISOLNUM(self) -> Any: ...
    @property
    def ISOWCR(self) -> Any: ...
    @property
    def ISWCR(self) -> Any: ...
    @property
    def ISWL(self) -> Any: ...
    @property
    def ISWLPC(self) -> Any: ...
    @property
    def ISWU(self) -> Any: ...
    @property
    def JFUNC(self) -> Any: ...
    @property
    def JFUNCR(self) -> Any: ...
    @property
    def KRNUM(self) -> Any: ...
    @property
    def KRNUMMF(self) -> Any: ...
    @property
    def LAB(self) -> Any: ...
    @property
    def LANGMPL(self) -> Any: ...
    @property
    def LANGMUIR(self) -> Any: ...
    @property
    def LANGSOLV(self) -> Any: ...
    @property
    def LCUNIT(self) -> Any: ...
    @property
    def LGR(self) -> Any: ...
    @property
    def LGRCOPY(self) -> Any: ...
    @property
    def LGRFREE(self) -> Any: ...
    @property
    def LGRLOCK(self) -> Any: ...
    @property
    def LGROFF(self) -> Any: ...
    @property
    def LGRON(self) -> Any: ...
    @property
    def LICENSES(self) -> Any: ...
    @property
    def LIFTOPT(self) -> Any: ...
    @property
    def LINCOM(self) -> Any: ...
    @property
    def LINKPERM(self) -> Any: ...
    @property
    def LIVEOIL(self) -> Any: ...
    @property
    def LKRO(self) -> Any: ...
    @property
    def LKRORG(self) -> Any: ...
    @property
    def LKRORW(self) -> Any: ...
    @property
    def LKRW(self) -> Any: ...
    @property
    def LKRWR(self) -> Any: ...
    @property
    def LOAD(self) -> Any: ...
    @property
    def LOWSALT(self) -> Any: ...
    @property
    def LPCW(self) -> Any: ...
    @property
    def LSALTFNC(self) -> Any: ...
    @property
    def LSLTWNUM(self) -> Any: ...
    @property
    def LSNUM(self) -> Any: ...
    @property
    def LSOGCR(self) -> Any: ...
    @property
    def LSOWCR(self) -> Any: ...
    @property
    def LSWCR(self) -> Any: ...
    @property
    def LSWL(self) -> Any: ...
    @property
    def LSWLPC(self) -> Any: ...
    @property
    def LSWU(self) -> Any: ...
    @property
    def LTOSIGMA(self) -> Any: ...
    @property
    def LWKRO(self) -> Any: ...
    @property
    def LWKRORG(self) -> Any: ...
    @property
    def LWKRORW(self) -> Any: ...
    @property
    def LWKRW(self) -> Any: ...
    @property
    def LWKRWR(self) -> Any: ...
    @property
    def LWPCW(self) -> Any: ...
    @property
    def LWSLTNUM(self) -> Any: ...
    @property
    def LWSNUM(self) -> Any: ...
    @property
    def LWSOGCR(self) -> Any: ...
    @property
    def LWSOWCR(self) -> Any: ...
    @property
    def LWSWCR(self) -> Any: ...
    @property
    def LWSWL(self) -> Any: ...
    @property
    def LWSWLPC(self) -> Any: ...
    @property
    def LWSWU(self) -> Any: ...
    @property
    def LX(self) -> Any: ...
    @property
    def LXFIN(self) -> Any: ...
    @property
    def LY(self) -> Any: ...
    @property
    def LYFIN(self) -> Any: ...
    @property
    def LZ(self) -> Any: ...
    @property
    def LZFIN(self) -> Any: ...
    @property
    def MAPAXES(self) -> Any: ...
    @property
    def MAPUNITS(self) -> Any: ...
    @property
    def MASSFLOW(self) -> Any: ...
    @property
    def MATCORR(self) -> Any: ...
    @property
    def MAXVALUE(self) -> Any: ...
    @property
    def MECH(self) -> Any: ...
    @property
    def MEMORY(self) -> Any: ...
    @property
    def MESSAGE(self) -> Any: ...
    @property
    def MESSAGES(self) -> Any: ...
    @property
    def MESSOPTS(self) -> Any: ...
    @property
    def MESSSRVC(self) -> Any: ...
    @property
    def METRIC(self) -> Any: ...
    @property
    def MICP(self) -> Any: ...
    @property
    def MINNNCT(self) -> Any: ...
    @property
    def MINNPCOL(self) -> Any: ...
    @property
    def MINPORV(self) -> Any: ...
    @property
    def MINPV(self) -> Any: ...
    @property
    def MINPVV(self) -> Any: ...
    @property
    def MINVALUE(self) -> Any: ...
    @property
    def MISC(self) -> Any: ...
    @property
    def MISCIBLE(self) -> Any: ...
    @property
    def MISCNUM(self) -> Any: ...
    @property
    def MLANG(self) -> Any: ...
    @property
    def MLANGSLV(self) -> Any: ...
    @property
    def MONITOR(self) -> Any: ...
    @property
    def MPFANUM(self) -> Any: ...
    @property
    def MPFNNC(self) -> Any: ...
    @property
    def MSFN(self) -> Any: ...
    @property
    def MSGFILE(self) -> Any: ...
    @property
    def MSUM_PROBE(self) -> Any: ...
    @property
    def MULSGGD(self) -> Any: ...
    @property
    def MULSGGDV(self) -> Any: ...
    @property
    def MULTFLT(self) -> Any: ...
    @property
    def MULTIN(self) -> Any: ...
    @property
    def MULTIPLY(self) -> Any: ...
    @property
    def MULTIREG(self) -> Any: ...
    @property
    def MULTNUM(self) -> Any: ...
    @property
    def MULTOUT(self) -> Any: ...
    @property
    def MULTOUTS(self) -> Any: ...
    @property
    def MULTPV(self) -> Any: ...
    @property
    def MULTREAL(self) -> Any: ...
    @property
    def MULTREGD(self) -> Any: ...
    @property
    def MULTREGH(self) -> Any: ...
    @property
    def MULTREGP(self) -> Any: ...
    @property
    def MULTREGT(self) -> Any: ...
    @property
    def MULTSIG(self) -> Any: ...
    @property
    def MULTSIGV(self) -> Any: ...
    @property
    def MULT_XYZ(self) -> Any: ...
    @property
    def MW(self) -> Any: ...
    @property
    def MWS(self) -> Any: ...
    @property
    def NARROW(self) -> Any: ...
    @property
    def NCOMPS(self) -> Any: ...
    @property
    def NCONSUMP(self) -> Any: ...
    @property
    def NEFAC(self) -> Any: ...
    @property
    def NETBALAN(self) -> Any: ...
    @property
    def NETCOMPA(self) -> Any: ...
    @property
    def NETWORK(self) -> Any: ...
    @property
    def NETWORK_PROBE(self) -> Any: ...
    @property
    def NEWTRAN(self) -> Any: ...
    @property
    def NEXTSTEP(self) -> Any: ...
    @property
    def NEXTSTPL(self) -> Any: ...
    @property
    def NINENUM(self) -> Any: ...
    @property
    def NINEPOIN(self) -> Any: ...
    @property
    def NMATOPTS(self) -> Any: ...
    @property
    def NMATRIX(self) -> Any: ...
    @property
    def NMESSAGE(self) -> Any: ...
    @property
    def NNC(self) -> Any: ...
    @property
    def NNEWTF(self) -> Any: ...
    @property
    def NOCASC(self) -> Any: ...
    @property
    def NODEPROP(self) -> Any: ...
    @property
    def NODPPM(self) -> Any: ...
    @property
    def NOECHO(self) -> Any: ...
    @property
    def NOGGF(self) -> Any: ...
    @property
    def NOGRAV(self) -> Any: ...
    @property
    def NOHMD(self) -> Any: ...
    @property
    def NOHMO(self) -> Any: ...
    @property
    def NOHYST(self) -> Any: ...
    @property
    def NOINSPEC(self) -> Any: ...
    @property
    def NOMONITO(self) -> Any: ...
    @property
    def NONNC(self) -> Any: ...
    @property
    def NORSSPEC(self) -> Any: ...
    @property
    def NOSIM(self) -> Any: ...
    @property
    def NOWARN(self) -> Any: ...
    @property
    def NOWARNEP(self) -> Any: ...
    @property
    def NRSOUT(self) -> Any: ...
    @property
    def NSTACK(self) -> Any: ...
    @property
    def NTG(self) -> Any: ...
    @property
    def NUMRES(self) -> Any: ...
    @property
    def NUPCOL(self) -> Any: ...
    @property
    def NWATREM(self) -> Any: ...
    @property
    def NXFIN(self) -> Any: ...
    @property
    def NYFIN(self) -> Any: ...
    @property
    def NZFIN(self) -> Any: ...
    @property
    def OCOMPIDX(self) -> Any: ...
    @property
    def OFM(self) -> Any: ...
    @property
    def OIL(self) -> Any: ...
    @property
    def OILAPI(self) -> Any: ...
    @property
    def OILCOMPR(self) -> Any: ...
    @property
    def OILDENT(self) -> Any: ...
    @property
    def OILJT(self) -> Any: ...
    @property
    def OILMW(self) -> Any: ...
    @property
    def OILVISCT(self) -> Any: ...
    @property
    def OILVTIM(self) -> Any: ...
    @property
    def OLDTRAN(self) -> Any: ...
    @property
    def OLDTRANR(self) -> Any: ...
    @property
    def OPERATE(self) -> Any: ...
    @property
    def OPERATER(self) -> Any: ...
    @property
    def OPERNUM(self) -> Any: ...
    @property
    def OPTIONS(self) -> Any: ...
    @property
    def OPTIONS3(self) -> Any: ...
    @property
    def OUTRAD(self) -> Any: ...
    @property
    def OUTSOL(self) -> Any: ...
    @property
    def OVERBURD(self) -> Any: ...
    @property
    def PARALLEL(self) -> Any: ...
    @property
    def PARAOPTS(self) -> Any: ...
    @property
    def PARTTRAC(self) -> Any: ...
    @property
    def PATHS(self) -> Any: ...
    @property
    def PBUB(self) -> Any: ...
    @property
    def PBVD(self) -> Any: ...
    @property
    def PCFACT(self) -> Any: ...
    @property
    def PCG(self) -> Any: ...
    @property
    def PCG32D(self) -> Any: ...
    @property
    def PCRIT(self) -> Any: ...
    @property
    def PCW(self) -> Any: ...
    @property
    def PCW32D(self) -> Any: ...
    @property
    def PDEW(self) -> Any: ...
    @property
    def PDVD(self) -> Any: ...
    @property
    def PEBI(self) -> Any: ...
    @property
    def PECOEFS(self) -> Any: ...
    @property
    def PEDIMS(self) -> Any: ...
    @property
    def PEGTABX(self) -> Any: ...
    @property
    def PEKTABX(self) -> Any: ...
    @property
    def PENUM(self) -> Any: ...
    @property
    def PERFORMANCE_PROBE(self) -> Any: ...
    @property
    def PERMAVE(self) -> Any: ...
    @property
    def PERMFACT(self) -> Any: ...
    @property
    def PERMJFUN(self) -> Any: ...
    @property
    def PERMR(self) -> Any: ...
    @property
    def PERMTHT(self) -> Any: ...
    @property
    def PERMX(self) -> Any: ...
    @property
    def PERMXY(self) -> Any: ...
    @property
    def PERMY(self) -> Any: ...
    @property
    def PERMYZ(self) -> Any: ...
    @property
    def PERMZ(self) -> Any: ...
    @property
    def PERMZX(self) -> Any: ...
    @property
    def PETGRID(self) -> Any: ...
    @property
    def PETOPTS(self) -> Any: ...
    @property
    def PICOND(self) -> Any: ...
    @property
    def PIMTDIMS(self) -> Any: ...
    @property
    def PIMULTAB(self) -> Any: ...
    @property
    def PINCH(self) -> Any: ...
    @property
    def PINCHNUM(self) -> Any: ...
    @property
    def PINCHOUT(self) -> Any: ...
    @property
    def PINCHREG(self) -> Any: ...
    @property
    def PINCHXY(self) -> Any: ...
    @property
    def PINTDIMS(self) -> Any: ...
    @property
    def PLMIXNUM(self) -> Any: ...
    @property
    def PLMIXPAR(self) -> Any: ...
    @property
    def PLYADS(self) -> Any: ...
    @property
    def PLYADSS(self) -> Any: ...
    @property
    def PLYATEMP(self) -> Any: ...
    @property
    def PLYCAMAX(self) -> Any: ...
    @property
    def PLYDHFLF(self) -> Any: ...
    @property
    def PLYESAL(self) -> Any: ...
    @property
    def PLYKRRF(self) -> Any: ...
    @property
    def PLYMAX(self) -> Any: ...
    @property
    def PLYMWINJ(self) -> Any: ...
    @property
    def PLYOPTS(self) -> Any: ...
    @property
    def PLYRMDEN(self) -> Any: ...
    @property
    def PLYROCK(self) -> Any: ...
    @property
    def PLYROCKM(self) -> Any: ...
    @property
    def PLYSHEAR(self) -> Any: ...
    @property
    def PLYSHLOG(self) -> Any: ...
    @property
    def PLYTRRF(self) -> Any: ...
    @property
    def PLYTRRFA(self) -> Any: ...
    @property
    def PLYVISC(self) -> Any: ...
    @property
    def PLYVISCS(self) -> Any: ...
    @property
    def PLYVISCT(self) -> Any: ...
    @property
    def PLYVMH(self) -> Any: ...
    @property
    def PLYVSCST(self) -> Any: ...
    @property
    def PMAX(self) -> Any: ...
    @property
    def PMISC(self) -> Any: ...
    @property
    def POELCOEF(self) -> Any: ...
    @property
    def POLYMER(self) -> Any: ...
    @property
    def POLYMW(self) -> Any: ...
    @property
    def PORO(self) -> Any: ...
    @property
    def PORV(self) -> Any: ...
    @property
    def PPCWMAX(self) -> Any: ...
    @property
    def PRATIO(self) -> Any: ...
    @property
    def PRECSALT(self) -> Any: ...
    @property
    def PREF(self) -> Any: ...
    @property
    def PREFS(self) -> Any: ...
    @property
    def PRESSURE(self) -> Any: ...
    @property
    def PRIORITY(self) -> Any: ...
    @property
    def PROPS(self) -> Any: ...
    @property
    def PRORDER(self) -> Any: ...
    @property
    def PRVD(self) -> Any: ...
    @property
    def PSTEADY(self) -> Any: ...
    @property
    def PSWRG(self) -> Any: ...
    @property
    def PSWRO(self) -> Any: ...
    @property
    def PVCDO(self) -> Any: ...
    @property
    def PVCO(self) -> Any: ...
    @property
    def PVDG(self) -> Any: ...
    @property
    def PVDO(self) -> Any: ...
    @property
    def PVDS(self) -> Any: ...
    @property
    def PVTG(self) -> Any: ...
    @property
    def PVTGW(self) -> Any: ...
    @property
    def PVTGWO(self) -> Any: ...
    @property
    def PVTNUM(self) -> Any: ...
    @property
    def PVTO(self) -> Any: ...
    @property
    def PVTSOL(self) -> Any: ...
    @property
    def PVTW(self) -> Any: ...
    @property
    def PVTWSALT(self) -> Any: ...
    @property
    def PVT_M(self) -> Any: ...
    @property
    def PVZG(self) -> Any: ...
    @property
    def PYACTION(self) -> Any: ...
    @property
    def PYINPUT(self) -> Any: ...
    @property
    def QDRILL(self) -> Any: ...
    @property
    def QHRATING(self) -> Any: ...
    @property
    def QMOBIL(self) -> Any: ...
    @property
    def RADFIN(self) -> Any: ...
    @property
    def RADFIN4(self) -> Any: ...
    @property
    def RADIAL(self) -> Any: ...
    @property
    def RAINFALL(self) -> Any: ...
    @property
    def RBEDCONT(self) -> Any: ...
    @property
    def RCMASTS(self) -> Any: ...
    @property
    def REACHES(self) -> Any: ...
    @property
    def READDATA(self) -> Any: ...
    @property
    def REFINE(self) -> Any: ...
    @property
    def REGDIMS(self) -> Any: ...
    @property
    def REGION2REGION_PROBE(self) -> Any: ...
    @property
    def REGION2REGION_PROBE_E300(self) -> Any: ...
    @property
    def REGIONS(self) -> Any: ...
    @property
    def REGION_PROBE(self) -> Any: ...
    @property
    def REGION_PROBE_OPM(self) -> Any: ...
    @property
    def RESIDNUM(self) -> Any: ...
    @property
    def RESTART(self) -> Any: ...
    @property
    def RESVNUM(self) -> Any: ...
    @property
    def RHO(self) -> Any: ...
    @property
    def RIVDEBUG(self) -> Any: ...
    @property
    def RIVERSYS(self) -> Any: ...
    @property
    def RIVRDIMS(self) -> Any: ...
    @property
    def RIVRPROP(self) -> Any: ...
    @property
    def RIVRXSEC(self) -> Any: ...
    @property
    def RIVSALT(self) -> Any: ...
    @property
    def RIVTRACE(self) -> Any: ...
    @property
    def RKTRMDIR(self) -> Any: ...
    @property
    def ROCK(self) -> Any: ...
    @property
    def ROCK2D(self) -> Any: ...
    @property
    def ROCK2DTR(self) -> Any: ...
    @property
    def ROCKCOMP(self) -> Any: ...
    @property
    def ROCKFRAC(self) -> Any: ...
    @property
    def ROCKNUM(self) -> Any: ...
    @property
    def ROCKOPTS(self) -> Any: ...
    @property
    def ROCKPAMA(self) -> Any: ...
    @property
    def ROCKTAB(self) -> Any: ...
    @property
    def ROCKTABH(self) -> Any: ...
    @property
    def ROCKTABW(self) -> Any: ...
    @property
    def ROCKTHSG(self) -> Any: ...
    @property
    def ROCKTSIG(self) -> Any: ...
    @property
    def ROCKV(self) -> Any: ...
    @property
    def ROCKWNOD(self) -> Any: ...
    @property
    def RPTCPL(self) -> Any: ...
    @property
    def RPTGRID(self) -> Any: ...
    @property
    def RPTGRIDL(self) -> Any: ...
    @property
    def RPTHMD(self) -> Any: ...
    @property
    def RPTHMG(self) -> Any: ...
    @property
    def RPTHMW(self) -> Any: ...
    @property
    def RPTINIT(self) -> Any: ...
    @property
    def RPTISOL(self) -> Any: ...
    @property
    def RPTONLY(self) -> Any: ...
    @property
    def RPTONLYO(self) -> Any: ...
    @property
    def RPTPROPS(self) -> Any: ...
    @property
    def RPTREGS(self) -> Any: ...
    @property
    def RPTRST(self) -> Any: ...
    @property
    def RPTRUNSP(self) -> Any: ...
    @property
    def RPTSCHED(self) -> Any: ...
    @property
    def RPTSMRY(self) -> Any: ...
    @property
    def RPTSOL(self) -> Any: ...
    @property
    def RS(self) -> Any: ...
    @property
    def RSCONST(self) -> Any: ...
    @property
    def RSCONSTT(self) -> Any: ...
    @property
    def RSGI(self) -> Any: ...
    @property
    def RSSPEC(self) -> Any: ...
    @property
    def RSVD(self) -> Any: ...
    @property
    def RSW(self) -> Any: ...
    @property
    def RTEMP(self) -> Any: ...
    @property
    def RTEMPA(self) -> Any: ...
    @property
    def RTEMPVD(self) -> Any: ...
    @property
    def RUNSPEC(self) -> Any: ...
    @property
    def RUNSUM(self) -> Any: ...
    @property
    def RV(self) -> Any: ...
    @property
    def RVCONST(self) -> Any: ...
    @property
    def RVCONSTT(self) -> Any: ...
    @property
    def RVGI(self) -> Any: ...
    @property
    def RVVD(self) -> Any: ...
    @property
    def RVW(self) -> Any: ...
    @property
    def RVWVD(self) -> Any: ...
    @property
    def RWGSALT(self) -> Any: ...
    @property
    def SALINITY(self) -> Any: ...
    @property
    def SALT(self) -> Any: ...
    @property
    def SALTMF(self) -> Any: ...
    @property
    def SALTNODE(self) -> Any: ...
    @property
    def SALTP(self) -> Any: ...
    @property
    def SALTPVD(self) -> Any: ...
    @property
    def SALTREST(self) -> Any: ...
    @property
    def SALTSOL(self) -> Any: ...
    @property
    def SALTVD(self) -> Any: ...
    @property
    def SAMG(self) -> Any: ...
    @property
    def SATNUM(self) -> Any: ...
    @property
    def SATOPTS(self) -> Any: ...
    @property
    def SAVE(self) -> Any: ...
    @property
    def SBIOF(self) -> Any: ...
    @property
    def SCALC(self) -> Any: ...
    @property
    def SCALECRS(self) -> Any: ...
    @property
    def SCALELIM(self) -> Any: ...
    @property
    def SCDATAB(self) -> Any: ...
    @property
    def SCDETAB(self) -> Any: ...
    @property
    def SCDPDIMS(self) -> Any: ...
    @property
    def SCDPTAB(self) -> Any: ...
    @property
    def SCDPTRAC(self) -> Any: ...
    @property
    def SCHEDULE(self) -> Any: ...
    @property
    def SCVD(self) -> Any: ...
    @property
    def SDENSITY(self) -> Any: ...
    @property
    def SEGMENT_PROBE(self) -> Any: ...
    @property
    def SEPARATE(self) -> Any: ...
    @property
    def SEPVALS(self) -> Any: ...
    @property
    def SFOAM(self) -> Any: ...
    @property
    def SGAS(self) -> Any: ...
    @property
    def SGCR(self) -> Any: ...
    @property
    def SGCWMIS(self) -> Any: ...
    @property
    def SGF32D(self) -> Any: ...
    @property
    def SGFN(self) -> Any: ...
    @property
    def SGL(self) -> Any: ...
    @property
    def SGLPC(self) -> Any: ...
    @property
    def SGOF(self) -> Any: ...
    @property
    def SGOFLET(self) -> Any: ...
    @property
    def SGU(self) -> Any: ...
    @property
    def SGWFN(self) -> Any: ...
    @property
    def SHRATE(self) -> Any: ...
    @property
    def SIGMA(self) -> Any: ...
    @property
    def SIGMAGDV(self) -> Any: ...
    @property
    def SIGMATH(self) -> Any: ...
    @property
    def SIGMAV(self) -> Any: ...
    @property
    def SIMULATE(self) -> Any: ...
    @property
    def SKIP(self) -> Any: ...
    @property
    def SKIP100(self) -> Any: ...
    @property
    def SKIP300(self) -> Any: ...
    @property
    def SKIPREST(self) -> Any: ...
    @property
    def SKPRPOLY(self) -> Any: ...
    @property
    def SKPRWAT(self) -> Any: ...
    @property
    def SKRO(self) -> Any: ...
    @property
    def SKRORG(self) -> Any: ...
    @property
    def SKRORW(self) -> Any: ...
    @property
    def SKRW(self) -> Any: ...
    @property
    def SKRWR(self) -> Any: ...
    @property
    def SLAVES(self) -> Any: ...
    @property
    def SLGOF(self) -> Any: ...
    @property
    def SMICR(self) -> Any: ...
    @property
    def SMRYDIMS(self) -> Any: ...
    @property
    def SMULTX(self) -> Any: ...
    @property
    def SMULTY(self) -> Any: ...
    @property
    def SMULTZ(self) -> Any: ...
    @property
    def SOCRS(self) -> Any: ...
    @property
    def SOF2(self) -> Any: ...
    @property
    def SOF3(self) -> Any: ...
    @property
    def SOF32D(self) -> Any: ...
    @property
    def SOGCR(self) -> Any: ...
    @property
    def SOIL(self) -> Any: ...
    @property
    def SOLID(self) -> Any: ...
    @property
    def SOLUTION(self) -> Any: ...
    @property
    def SOLVCONC(self) -> Any: ...
    @property
    def SOLVDIMS(self) -> Any: ...
    @property
    def SOLVDIRS(self) -> Any: ...
    @property
    def SOLVENT(self) -> Any: ...
    @property
    def SOLVFRAC(self) -> Any: ...
    @property
    def SOLVNUM(self) -> Any: ...
    @property
    def SOMGAS(self) -> Any: ...
    @property
    def SOMWAT(self) -> Any: ...
    @property
    def SORWMIS(self) -> Any: ...
    @property
    def SOURCE(self) -> Any: ...
    @property
    def SOWCR(self) -> Any: ...
    @property
    def SOXYG(self) -> Any: ...
    @property
    def SPECGRID(self) -> Any: ...
    @property
    def SPECHEAT(self) -> Any: ...
    @property
    def SPECROCK(self) -> Any: ...
    @property
    def SPIDER(self) -> Any: ...
    @property
    def SPOLY(self) -> Any: ...
    @property
    def SPOLYMW(self) -> Any: ...
    @property
    def SSFN(self) -> Any: ...
    @property
    def SSGCR(self) -> Any: ...
    @property
    def SSGL(self) -> Any: ...
    @property
    def SSOGCR(self) -> Any: ...
    @property
    def SSOL(self) -> Any: ...
    @property
    def SSOWCR(self) -> Any: ...
    @property
    def SSWCR(self) -> Any: ...
    @property
    def SSWL(self) -> Any: ...
    @property
    def SSWU(self) -> Any: ...
    @property
    def START(self) -> Any: ...
    @property
    def STCOND(self) -> Any: ...
    @property
    def STOG(self) -> Any: ...
    @property
    def STONE(self) -> Any: ...
    @property
    def STONE1(self) -> Any: ...
    @property
    def STONE1EX(self) -> Any: ...
    @property
    def STONE2(self) -> Any: ...
    @property
    def STOW(self) -> Any: ...
    @property
    def STREQUIL(self) -> Any: ...
    @property
    def STRESSEQUILNUM(self) -> Any: ...
    @property
    def STWG(self) -> Any: ...
    @property
    def SUMMARY(self) -> Any: ...
    @property
    def SUMTHIN(self) -> Any: ...
    @property
    def SUREA(self) -> Any: ...
    @property
    def SURF(self) -> Any: ...
    @property
    def SURFACT(self) -> Any: ...
    @property
    def SURFACTW(self) -> Any: ...
    @property
    def SURFADDW(self) -> Any: ...
    @property
    def SURFADS(self) -> Any: ...
    @property
    def SURFCAPD(self) -> Any: ...
    @property
    def SURFESAL(self) -> Any: ...
    @property
    def SURFNUM(self) -> Any: ...
    @property
    def SURFOPTS(self) -> Any: ...
    @property
    def SURFROCK(self) -> Any: ...
    @property
    def SURFST(self) -> Any: ...
    @property
    def SURFSTES(self) -> Any: ...
    @property
    def SURFVISC(self) -> Any: ...
    @property
    def SURFWNUM(self) -> Any: ...
    @property
    def SWAT(self) -> Any: ...
    @property
    def SWATINIT(self) -> Any: ...
    @property
    def SWCR(self) -> Any: ...
    @property
    def SWF32D(self) -> Any: ...
    @property
    def SWFN(self) -> Any: ...
    @property
    def SWINGFAC(self) -> Any: ...
    @property
    def SWL(self) -> Any: ...
    @property
    def SWLPC(self) -> Any: ...
    @property
    def SWOF(self) -> Any: ...
    @property
    def SWOFLET(self) -> Any: ...
    @property
    def SWU(self) -> Any: ...
    @property
    def TABDIMS(self) -> Any: ...
    @property
    def TBLK(self) -> Any: ...
    @property
    def TCRIT(self) -> Any: ...
    @property
    def TEMP(self) -> Any: ...
    @property
    def TEMPI(self) -> Any: ...
    @property
    def TEMPNODE(self) -> Any: ...
    @property
    def TEMPTVD(self) -> Any: ...
    @property
    def TEMPVD(self) -> Any: ...
    @property
    def THCGAS(self) -> Any: ...
    @property
    def THCO2MIX(self) -> Any: ...
    @property
    def THCOIL(self) -> Any: ...
    @property
    def THCONR(self) -> Any: ...
    @property
    def THCONSF(self) -> Any: ...
    @property
    def THCROCK(self) -> Any: ...
    @property
    def THCWATER(self) -> Any: ...
    @property
    def THELCOEF(self) -> Any: ...
    @property
    def THERMAL(self) -> Any: ...
    @property
    def THERMEXR(self) -> Any: ...
    @property
    def THPRES(self) -> Any: ...
    @property
    def THPRESFT(self) -> Any: ...
    @property
    def TIGHTEN(self) -> Any: ...
    @property
    def TIGHTENP(self) -> Any: ...
    @property
    def TIME(self) -> Any: ...
    @property
    def TITLE(self) -> Any: ...
    @property
    def TLMIXPAR(self) -> Any: ...
    @property
    def TLPMIXPA(self) -> Any: ...
    @property
    def TNUM(self) -> Any: ...
    @property
    def TOLCRIT(self) -> Any: ...
    @property
    def TOPS(self) -> Any: ...
    @property
    def TPAMEPS(self) -> Any: ...
    @property
    def TPAMEPSS(self) -> Any: ...
    @property
    def TRACER(self) -> Any: ...
    @property
    def TRACERKM(self) -> Any: ...
    @property
    def TRACERKP(self) -> Any: ...
    @property
    def TRACERS(self) -> Any: ...
    @property
    def TRACITVD(self) -> Any: ...
    @property
    def TRACTVD(self) -> Any: ...
    @property
    def TRADS(self) -> Any: ...
    @property
    def TRANGL(self) -> Any: ...
    @property
    def TRANR(self) -> Any: ...
    @property
    def TRANTHT(self) -> Any: ...
    @property
    def TRANX(self) -> Any: ...
    @property
    def TRANY(self) -> Any: ...
    @property
    def TRANZ(self) -> Any: ...
    @property
    def TRDCY(self) -> Any: ...
    @property
    def TRDIF(self) -> Any: ...
    @property
    def TRDIS(self) -> Any: ...
    @property
    def TREF(self) -> Any: ...
    @property
    def TREFS(self) -> Any: ...
    @property
    def TRKPF(self) -> Any: ...
    @property
    def TRNHD(self) -> Any: ...
    @property
    def TRPLPORO(self) -> Any: ...
    @property
    def TRROCK(self) -> Any: ...
    @property
    def TSTEP(self) -> Any: ...
    @property
    def TUNING(self) -> Any: ...
    @property
    def TUNINGDP(self) -> Any: ...
    @property
    def TUNINGH(self) -> Any: ...
    @property
    def TUNINGL(self) -> Any: ...
    @property
    def TUNINGS(self) -> Any: ...
    @property
    def TVDP(self) -> Any: ...
    @property
    def TZONE(self) -> Any: ...
    @property
    def UDADIMS(self) -> Any: ...
    @property
    def UDQ(self) -> Any: ...
    @property
    def UDQDIMS(self) -> Any: ...
    @property
    def UDQPARAM(self) -> Any: ...
    @property
    def UDT(self) -> Any: ...
    @property
    def UDTDIMS(self) -> Any: ...
    @property
    def UNCODHMD(self) -> Any: ...
    @property
    def UNIFIN(self) -> Any: ...
    @property
    def UNIFOUT(self) -> Any: ...
    @property
    def UNIFOUTS(self) -> Any: ...
    @property
    def UNIFSAVE(self) -> Any: ...
    @property
    def USECUPL(self) -> Any: ...
    @property
    def USEFLUX(self) -> Any: ...
    @property
    def USENOFLO(self) -> Any: ...
    @property
    def VAPOIL(self) -> Any: ...
    @property
    def VAPPARS(self) -> Any: ...
    @property
    def VAPWAT(self) -> Any: ...
    @property
    def VCRIT(self) -> Any: ...
    @property
    def VDFLOW(self) -> Any: ...
    @property
    def VDFLOWR(self) -> Any: ...
    @property
    def VE(self) -> Any: ...
    @property
    def VEDEBUG(self) -> Any: ...
    @property
    def VEFIN(self) -> Any: ...
    @property
    def VEFRAC(self) -> Any: ...
    @property
    def VEFRACP(self) -> Any: ...
    @property
    def VEFRACPV(self) -> Any: ...
    @property
    def VEFRACV(self) -> Any: ...
    @property
    def VFPCHK(self) -> Any: ...
    @property
    def VFPIDIMS(self) -> Any: ...
    @property
    def VFPINJ(self) -> Any: ...
    @property
    def VFPPDIMS(self) -> Any: ...
    @property
    def VFPPROD(self) -> Any: ...
    @property
    def VFPTABL(self) -> Any: ...
    @property
    def VISAGE(self) -> Any: ...
    @property
    def VISCAQA(self) -> Any: ...
    @property
    def VISCD(self) -> Any: ...
    @property
    def VISCREF(self) -> Any: ...
    @property
    def VISDATES(self) -> Any: ...
    @property
    def VISOPTS(self) -> Any: ...
    @property
    def WAGHYSTR(self) -> Any: ...
    @property
    def WAITBAL(self) -> Any: ...
    @property
    def WALKALIN(self) -> Any: ...
    @property
    def WALQCALC(self) -> Any: ...
    @property
    def WAPI(self) -> Any: ...
    @property
    def WARN(self) -> Any: ...
    @property
    def WATDENT(self) -> Any: ...
    @property
    def WATER(self) -> Any: ...
    @property
    def WATJT(self) -> Any: ...
    @property
    def WATVISCT(self) -> Any: ...
    @property
    def WBHGLR(self) -> Any: ...
    @property
    def WBOREVOL(self) -> Any: ...
    @property
    def WCALCVAL(self) -> Any: ...
    @property
    def WCONHIST(self) -> Any: ...
    @property
    def WCONINJ(self) -> Any: ...
    @property
    def WCONINJE(self) -> Any: ...
    @property
    def WCONINJH(self) -> Any: ...
    @property
    def WCONINJP(self) -> Any: ...
    @property
    def WCONPROD(self) -> Any: ...
    @property
    def WCUTBACK(self) -> Any: ...
    @property
    def WCUTBACT(self) -> Any: ...
    @property
    def WCYCLE(self) -> Any: ...
    @property
    def WDFAC(self) -> Any: ...
    @property
    def WDFACCOR(self) -> Any: ...
    @property
    def WDRILPRI(self) -> Any: ...
    @property
    def WDRILRES(self) -> Any: ...
    @property
    def WDRILTIM(self) -> Any: ...
    @property
    def WECON(self) -> Any: ...
    @property
    def WECONINJ(self) -> Any: ...
    @property
    def WECONT(self) -> Any: ...
    @property
    def WEFAC(self) -> Any: ...
    @property
    def WELCNTL(self) -> Any: ...
    @property
    def WELDEBUG(self) -> Any: ...
    @property
    def WELDRAW(self) -> Any: ...
    @property
    def WELEVNT(self) -> Any: ...
    @property
    def WELLDIMS(self) -> Any: ...
    @property
    def WELLSTRE(self) -> Any: ...
    @property
    def WELL_COMPLETION_PROBE(self) -> Any: ...
    @property
    def WELL_PROBE(self) -> Any: ...
    @property
    def WELL_PROBE_COMP(self) -> Any: ...
    @property
    def WELL_PROBE_OPM(self) -> Any: ...
    @property
    def WELMOVEL(self) -> Any: ...
    @property
    def WELOPEN(self) -> Any: ...
    @property
    def WELOPENL(self) -> Any: ...
    @property
    def WELPI(self) -> Any: ...
    @property
    def WELPRI(self) -> Any: ...
    @property
    def WELSEGS(self) -> Any: ...
    @property
    def WELSOMIN(self) -> Any: ...
    @property
    def WELSPECL(self) -> Any: ...
    @property
    def WELSPECS(self) -> Any: ...
    @property
    def WELTARG(self) -> Any: ...
    @property
    def WELTRAJ(self) -> Any: ...
    @property
    def WFOAM(self) -> Any: ...
    @property
    def WFRICSEG(self) -> Any: ...
    @property
    def WFRICSGL(self) -> Any: ...
    @property
    def WFRICTN(self) -> Any: ...
    @property
    def WFRICTNL(self) -> Any: ...
    @property
    def WGASPROD(self) -> Any: ...
    @property
    def WGORPEN(self) -> Any: ...
    @property
    def WGRUPCON(self) -> Any: ...
    @property
    def WH2NUM(self) -> Any: ...
    @property
    def WH3NUM(self) -> Any: ...
    @property
    def WHEDREFD(self) -> Any: ...
    @property
    def WHISTCTL(self) -> Any: ...
    @property
    def WHTEMP(self) -> Any: ...
    @property
    def WINJCLN(self) -> Any: ...
    @property
    def WINJDAM(self) -> Any: ...
    @property
    def WINJFCNC(self) -> Any: ...
    @property
    def WINJGAS(self) -> Any: ...
    @property
    def WINJMULT(self) -> Any: ...
    @property
    def WINJTEMP(self) -> Any: ...
    @property
    def WLIFT(self) -> Any: ...
    @property
    def WLIFTOPT(self) -> Any: ...
    @property
    def WLIMTOL(self) -> Any: ...
    @property
    def WLIST(self) -> Any: ...
    @property
    def WLISTARG(self) -> Any: ...
    @property
    def WLISTNAM(self) -> Any: ...
    @property
    def WMICP(self) -> Any: ...
    @property
    def WNETCTRL(self) -> Any: ...
    @property
    def WNETDP(self) -> Any: ...
    @property
    def WORKLIM(self) -> Any: ...
    @property
    def WORKTHP(self) -> Any: ...
    @property
    def WPAVE(self) -> Any: ...
    @property
    def WPAVEDEP(self) -> Any: ...
    @property
    def WPIMULT(self) -> Any: ...
    @property
    def WPIMULTL(self) -> Any: ...
    @property
    def WPITAB(self) -> Any: ...
    @property
    def WPLUG(self) -> Any: ...
    @property
    def WPMITAB(self) -> Any: ...
    @property
    def WPOLYMER(self) -> Any: ...
    @property
    def WPOLYRED(self) -> Any: ...
    @property
    def WPOTCALC(self) -> Any: ...
    @property
    def WREGROUP(self) -> Any: ...
    @property
    def WRFT(self) -> Any: ...
    @property
    def WRFTPLT(self) -> Any: ...
    @property
    def WSALT(self) -> Any: ...
    @property
    def WSCCLEAN(self) -> Any: ...
    @property
    def WSCCLENL(self) -> Any: ...
    @property
    def WSCTAB(self) -> Any: ...
    @property
    def WSEED(self) -> Any: ...
    @property
    def WSEGAICD(self) -> Any: ...
    @property
    def WSEGDFIN(self) -> Any: ...
    @property
    def WSEGDFMD(self) -> Any: ...
    @property
    def WSEGDFPA(self) -> Any: ...
    @property
    def WSEGDIMS(self) -> Any: ...
    @property
    def WSEGEXSS(self) -> Any: ...
    @property
    def WSEGFLIM(self) -> Any: ...
    @property
    def WSEGFMOD(self) -> Any: ...
    @property
    def WSEGINIT(self) -> Any: ...
    @property
    def WSEGITER(self) -> Any: ...
    @property
    def WSEGLABY(self) -> Any: ...
    @property
    def WSEGLINK(self) -> Any: ...
    @property
    def WSEGMULT(self) -> Any: ...
    @property
    def WSEGPROP(self) -> Any: ...
    @property
    def WSEGSEP(self) -> Any: ...
    @property
    def WSEGSICD(self) -> Any: ...
    @property
    def WSEGSOLV(self) -> Any: ...
    @property
    def WSEGTABL(self) -> Any: ...
    @property
    def WSEGVALV(self) -> Any: ...
    @property
    def WSF(self) -> Any: ...
    @property
    def WSKPTAB(self) -> Any: ...
    @property
    def WSOLVENT(self) -> Any: ...
    @property
    def WSURFACT(self) -> Any: ...
    @property
    def WTADD(self) -> Any: ...
    @property
    def WTEMP(self) -> Any: ...
    @property
    def WTEMPQ(self) -> Any: ...
    @property
    def WTEST(self) -> Any: ...
    @property
    def WTHPMAX(self) -> Any: ...
    @property
    def WTMULT(self) -> Any: ...
    @property
    def WTRACER(self) -> Any: ...
    @property
    def WVFPDP(self) -> Any: ...
    @property
    def WVFPEXP(self) -> Any: ...
    @property
    def WWPAVE(self) -> Any: ...
    @property
    def XMF(self) -> Any: ...
    @property
    def YMF(self) -> Any: ...
    @property
    def YMODULE(self) -> Any: ...
    @property
    def ZCORN(self) -> Any: ...
    @property
    def ZFACT1(self) -> Any: ...
    @property
    def ZFACT1S(self) -> Any: ...
    @property
    def ZFACTOR(self) -> Any: ...
    @property
    def ZFACTORS(self) -> Any: ...
    @property
    def ZIPP2OFF(self) -> Any: ...
    @property
    def ZIPPY2(self) -> Any: ...
    @property
    def ZMF(self) -> Any: ...
    @property
    def ZMFVD(self) -> Any: ...

class Connection:
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def attached_to_segment(self) -> bool: ...
    @property
    def center_depth(self) -> float: ...
    @property
    def cf(self) -> float: ...
    @property
    def complnum(self) -> int: ...
    @property
    def direction(self) -> str: ...
    @property
    def i(self) -> int: ...
    @property
    def j(self) -> int: ...
    @property
    def k(self) -> int: ...
    @property
    def kh(self) -> float: ...
    @property
    def number(self) -> int: ...
    @property
    def pos(self) -> Tuple[int,int,int]: ...
    @property
    def rw(self) -> float: ...
    @property
    def sat_table_id(self) -> int: ...
    @property
    def segment_number(self) -> int: ...
    @property
    def state(self) -> str: ...

class Deck:
    def __init__(self, *args, **kwargs) -> None: ...
    def active_unit_system(self, *args, **kwargs) -> Any: ...
    def add(self, keyword: DeckKeyword) -> None: ...
    def count(self, keyword: str) -> int: ...
    def default_unit_system(self, *args, **kwargs) -> Any: ...
    def __contains__(self, keyword: str) -> bool: ...
    @overload
    def __getitem__(self, index: int) -> DeckKeyword: ...
    @overload
    def __getitem__(self, keyword: str) -> DeckKeyword: ...
    @overload
    def __getitem__(self, keyword_index: tuple) -> DeckKeyword: ...
    def __iter__(self) -> Iterator: ...
    def __len__(self) -> int: ...

class DeckItem:
    def __init__(self, *args, **kwargs) -> None: ...
    def __defaulted(self, index: int) -> bool: ...
    def __has_value(self, index: int) -> bool: ...
    def __is_numeric(self) -> bool: ...
    def __uda_double(self) -> float: ...
    def __uda_str(self) -> str: ...
    def get_SI(self, index: int) -> float: ...
    def get_SI_data_list(self) -> list: ...
    def get_data_list(self) -> list: ...
    def get_int(self, index: int) -> int: ...
    def get_raw(self, index: int) -> float: ...
    def get_raw_data_list(self) -> list: ...
    def get_str(self, index: int) -> str: ...
    def get_uda(self, *args, **kwargs) -> Any: ...
    def is_double(self) -> bool: ...
    def is_int(self) -> bool: ...
    def is_string(self) -> bool: ...
    def is_uda(self) -> bool: ...
    def name(self) -> str: ...
    def __len__(self) -> int: ...

class DeckKeyword:
    @overload
    def __init__(self, parser_keyword) -> None: ...
    @overload
    def __init__(self, parser_keyword, record_list: list, active_system, default_system) -> None: ...
    @overload
    def __init__(self, parser_keyword, py_data: numpy.ndarray[numpy.int32]) -> None: ...
    @overload
    def __init__(self, parser_keyword, py_data: numpy.ndarray[numpy.float64], active_system, default_system) -> None: ...
    def get_SI_array(self) -> numpy.ndarray[numpy.float64]: ...
    def get_int_array(self) -> numpy.ndarray[numpy.int32]: ...
    def get_raw_array(self) -> numpy.ndarray[numpy.float64]: ...
    def __getitem__(self, index) -> Any: ...
    def __iter__(self) -> Iterator: ...
    def __len__(self) -> int: ...
    @property
    def name(self) -> str: ...

class DeckRecord:
    def __init__(self, *args, **kwargs) -> None: ...
    def __getitem__(self, index) -> Any: ...
    def __iter__(self) -> Iterator: ...
    def __len__(self) -> int: ...

class Dimension:
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def offset(self) -> float: ...
    @property
    def scaling(self) -> float: ...

class EGrid:
    def __init__(self, filename: str, grid_name: str = ...) -> None: ...
    def active_index(self, i: int, j: int, k: int) -> int: ...
    @overload
    def cellvolumes(self) -> numpy.ndarray: ...
    @overload
    def cellvolumes(self, mask: List[int]) -> numpy.ndarray: ...
    def export_mapaxes(self) -> List[float[6]]: ...
    def global_index(self, i: int, j: int, k: int) -> int: ...
    def ijk_from_active_index(self, active_index: int) -> List[int[3]]: ...
    def ijk_from_global_index(self, global_index: int) -> List[int[3]]: ...
    @overload
    def xyz_from_active_index(self, active_index: int) -> Tuple[List[float[8]],List[float[8]],List[float[8]]]: ...
    @overload
    def xyz_from_active_index(self, active_index: int, apply_mapaxes: bool) -> Tuple[List[float[8]],List[float[8]],List[float[8]]]: ...
    @overload
    def xyz_from_ijk(self, i: int, j: int, k: int) -> Tuple[List[float[8]],List[float[8]],List[float[8]]]: ...
    @overload
    def xyz_from_ijk(self, i: int, j: int, k: int, apply_mapaxes: bool) -> Tuple[List[float[8]],List[float[8]],List[float[8]]]: ...
    @property
    def active_cells(self) -> int: ...
    @property
    def dimension(self) -> List[int[3]]: ...

class EModel:
    def __init__(self, filename: str) -> None: ...
    @overload
    def __add_filter(self, key: str, operator: str, value: int) -> None: ...
    @overload
    def __add_filter(self, key: str, operator: str, value: float) -> None: ...
    @overload
    def __add_filter(self, key: str, operator: str, value1: int, value2: int) -> None: ...
    @overload
    def __add_filter(self, key: str, operator: str, value1: float, value2: float) -> None: ...
    def active_cells(self) -> int: ...
    def active_report_step(self) -> int: ...
    def add_hc_filter(self) -> None: ...
    def get(self, key: str) -> numpy.ndarray: ...
    def get_list_of_arrays(self) -> List[Tuple[str,eclArrType]]: ...
    def get_report_steps(self) -> List[int]: ...
    def grid_dims(self) -> Tuple[int,int,int]: ...
    def has_report_step(self, rstep: int) -> bool: ...
    def reset_filter(self) -> None: ...
    def set_depth_fwl(self, fwl: List[float]) -> None: ...
    def set_report_step(self, rstep: int) -> None: ...
    def __contains__(self, parameter: str) -> bool: ...

class ERft:
    def __init__(self, filename: str) -> None: ...
    @overload
    def __get_data(self, array_name: str, well_name: str, year: int, month: int, day: int) -> Tuple[numpy.ndarray,eclArrType]: ...
    @overload
    def __get_data(self, array_name: str, report_index: int) -> Tuple[numpy.ndarray,eclArrType]: ...
    @overload
    def __get_list_of_arrays(self, report_index: int) -> List[Tuple[str,eclArrType,int]]: ...
    @overload
    def __get_list_of_arrays(self, well_name: str, year: int, month: int, day: int) -> List[Tuple[str,eclArrType,int]]: ...
    @overload
    def __has_array(self, array_name: str, report_index: int) -> bool: ...
    @overload
    def __has_array(self, array_name: str, well_name: str, date: Tuple[int,int,int]) -> bool: ...
    def __has_rft(self, well_name: str, year: int, month: int, day: int) -> bool: ...
    def __len__(self) -> int: ...
    @property
    def list_of_rfts(self) -> List[Tuple[str,Tuple[int,int,int],float]]: ...

class ERst:
    def __init__(self, filename: str) -> None: ...
    def __contains(self, tuple: Tuple[str,int]) -> bool: ...
    @overload
    def __get_data(self, index: int, report_step: int) -> Tuple[numpy.ndarray,eclArrType]: ...
    @overload
    def __get_data(self, name: str, report_step: int, occurrence: int) -> Tuple[numpy.ndarray,eclArrType]: ...
    def __has_report_step(self, report_step: int) -> bool: ...
    @overload
    def arrays(self, report_step: int) -> List[Tuple[str,eclArrType,int]]: ...
    @overload
    def arrays(self, report_step: int, lgr_name: str) -> List[Tuple[str,eclArrType,int]]: ...
    def count(self, name: str, report_step: int) -> int: ...
    def load_report_step(self, report_step: int) -> None: ...
    def __len__(self) -> int: ...
    @property
    def report_steps(self) -> List[int]: ...

class ESmry:
    def __init__(self, filename: str, load_base_run: bool = ...) -> None: ...
    def __get_all(self, key: str) -> numpy.ndarray: ...
    def __get_at_rstep(self, key: str) -> numpy.ndarray: ...
    def __start_date(self) -> Tuple[int,int,int,int,int,int,bool]: ...
    def dates(self) -> List[datetime.datetime]: ...
    @overload
    def keys(self) -> List[str]: ...
    @overload
    def keys(self, pattern: str) -> List[str]: ...
    def make_esmry_file(self) -> None: ...
    def units(self, field: str) -> str: ...
    def __contains__(self, key: str) -> bool: ...
    def __len__(self) -> int: ...

class EclFile:
    def __init__(self, filename: str, preload: bool = ...) -> None: ...
    @overload
    def __get_data(self, index: int) -> Tuple[numpy.ndarray,eclArrType]: ...
    @overload
    def __get_data(self, name: str) -> Tuple[numpy.ndarray,eclArrType]: ...
    @overload
    def __get_data(self, name: str, occurrence: int) -> Tuple[numpy.ndarray,eclArrType]: ...
    def count(self, name: str) -> int: ...
    def __contains__(self, name: str) -> bool: ...
    def __len__(self) -> int: ...
    @property
    def arrays(self) -> List[Tuple[str,eclArrType,int]]: ...

class EclOutput:
    def __init__(self, filename: str, formatted: bool = ..., append: bool = ...) -> None: ...
    def __write_c0nn_array(self, array_name: str, data: List[str], element_size: int) -> None: ...
    def __write_char_array(self, array_name: str, data: List[str]) -> None: ...
    def __write_doub_array(self, array_name: str, data: List[float]) -> None: ...
    def __write_inte_array(self, array_name: str, data: List[int]) -> None: ...
    def __write_logi_array(self, array_name: str, data: List[bool]) -> None: ...
    def __write_real_array(self, array_name: str, data: List[float]) -> None: ...
    def write_message(self, msg: str) -> None: ...

class EclipseConfig:
    def __init__(self, *args, **kwargs) -> None: ...
    def init(self, *args, **kwargs) -> Any: ...

class EclipseGrid:
    def __init__(self, *args, **kwargs) -> None: ...
    def _getXYZ(self) -> tuple: ...
    @overload
    def getCellDepth(self, g: int) -> float: ...
    @overload
    def getCellDepth(self, i: int, j: int, k: int) -> float: ...
    @overload
    def getCellDepth(self) -> numpy.ndarray: ...
    @overload
    def getCellDepth(self, mask: List[int]) -> numpy.ndarray: ...
    @overload
    def getCellVolume(self, g: int) -> float: ...
    @overload
    def getCellVolume(self, i: int, j: int, k: int) -> float: ...
    @overload
    def getCellVolume(self) -> numpy.ndarray: ...
    @overload
    def getCellVolume(self, mask: List[int]) -> numpy.ndarray: ...
    def getIJK(self, g: int) -> tuple: ...
    def globalIndex(self, i: int, j: int, k: int) -> int: ...
    @property
    def cartesianSize(self) -> int: ...
    @property
    def nactive(self) -> int: ...
    @property
    def nx(self) -> int: ...
    @property
    def ny(self) -> int: ...
    @property
    def nz(self) -> int: ...

class EclipseState:
    def __init__(self, deck: Deck) -> None: ...
    def config(self) -> EclipseConfig: ...
    def faultFaces(self, fault_name: str) -> list: ...
    def faultNames(self) -> list: ...
    def field_props(self) -> FieldProperties: ...
    def grid(self) -> EclipseGrid: ...
    def has_input_nnc(self) -> bool: ...
    def input_nnc(self) -> list: ...
    def jfunc(self) -> dict: ...
    def simulation(self) -> SimulationConfig: ...
    def tables(self) -> Tables: ...
    @property
    def title(self) -> str: ...

class FieldProperties:
    def __init__(self, *args, **kwargs) -> None: ...
    def get_double_array(self, keyword: str) -> numpy.ndarray[numpy.float64]: ...
    def get_int_array(self, keyword: str) -> numpy.ndarray[numpy.int32]: ...
    def __contains__(self, keyword: str) -> bool: ...
    def __getitem__(self, keyword: str) -> numpy.ndarray: ...

class Group:
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def num_wells(self) -> int: ...
    @property
    def well_names(self) -> List[str]: ...

class IOConfig:
    def __init__(self, *args, **kwargs) -> None: ...

class InitConfig:
    def __init__(self, *args, **kwargs) -> None: ...
    def getRestartStep(self) -> int: ...
    def hasEquil(self) -> bool: ...
    def restartRequested(self) -> bool: ...

class Logger:
    def __init__(self, *args, **kwargs) -> None: ...

class OpmLog:
    def __init__(self, *args, **kwargs) -> None: ...
    def _setLogger(self, *args, **kwargs) -> Any: ...
    def bug(self, *args, **kwargs) -> Any: ...
    def debug(self, *args, **kwargs) -> Any: ...
    def error(self, *args, **kwargs) -> Any: ...
    def info(self, *args, **kwargs) -> Any: ...
    def note(self, *args, **kwargs) -> Any: ...
    def problem(self, *args, **kwargs) -> Any: ...
    def warning(self, *args, **kwargs) -> Any: ...

class ParseContext:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, action_list) -> None: ...
    def ignore_keyword(self, keyword: str) -> None: ...
    def update(self, keyword: str, action) -> None: ...

class Parser:
    def __init__(self, add_default: bool = ...) -> None: ...
    @overload
    def add_keyword(self, keyword: ParserKeyword) -> None: ...
    @overload
    def add_keyword(self, keyword_json: str) -> None: ...
    @overload
    def parse(self, filename: str) -> Deck: ...
    @overload
    def parse(self, filename: str, context: ParseContext) -> Deck: ...
    @overload
    def parse(self, filename: str, context: ParseContext, sections: List[eclSectionType]) -> Deck: ...
    @overload
    def parse_string(self, data: str) -> Deck: ...
    @overload
    def parse_string(self, data: str, context: ParseContext) -> Deck: ...
    def __getitem__(self, keyword: str) -> ParserKeyword: ...

class ParserKeyword:
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def name(self) -> str: ...

class Schedule:
    def __init__(self, deck: Deck, eclipse_state) -> None: ...
    def _groups(self, report_step: int) -> List[Group]: ...
    def get_injection_properties(self, well_name: str, report_step: int) -> Dict[str,float]: ...
    def get_production_properties(self, well_name: str, report_step: int) -> Dict[str,float]: ...
    def get_well(self, well_name: str, report_step: int) -> Well: ...
    def get_wells(self, report_step: int) -> List[Well]: ...
    @overload
    def insert_keywords(self, keywords: list, step: int) -> None: ...
    @overload
    def insert_keywords(self, data: str, step: int, unit_system: UnitSystem) -> None: ...
    @overload
    def insert_keywords(self, data: str, step: int) -> None: ...
    @overload
    def insert_keywords(self, data: str) -> None: ...
    @overload
    def open_well(self, well_name: str, step: int) -> None: ...
    @overload
    def open_well(self, well_name: str) -> None: ...
    @overload
    def shut_well(self, well_name: str, step: int) -> None: ...
    @overload
    def shut_well(self, well_name: str) -> None: ...
    @overload
    def stop_well(self, well_name: str, step: int) -> None: ...
    @overload
    def stop_well(self, well_name: str) -> None: ...
    def well_names(self, well_name_pattern: str) -> List[str]: ...
    def __contains__(self, well_name: str) -> bool: ...
    def __getitem__(self, report_step: int) -> ScheduleState: ...
    def __len__(self) -> int: ...
    @property
    def end(self) -> datetime.datetime: ...
    @property
    def reportsteps(self) -> List[datetime.datetime]: ...
    @property
    def start(self) -> datetime.datetime: ...
    @property
    def timesteps(self) -> List[datetime.datetime]: ...

class ScheduleState:
    def __init__(self, *args, **kwargs) -> None: ...
    def group(self, group_name: str) -> Group: ...
    @property
    def nupcol(self) -> int: ...

class SimulationConfig:
    def __init__(self, *args, **kwargs) -> None: ...
    def hasDISGAS(self) -> bool: ...
    def hasDISGASW(self) -> bool: ...
    def hasThresholdPressure(self) -> bool: ...
    def hasVAPOIL(self) -> bool: ...
    def hasVAPWAT(self) -> bool: ...
    def useCPR(self) -> bool: ...
    def useNONNC(self) -> bool: ...

class SummaryConfig:
    def __init__(self, arg0: Deck, arg1, arg2) -> None: ...
    def __contains__(self, arg0: str) -> bool: ...

class SummaryState:
    def __init__(self, arg0: int) -> None: ...
    def elapsed(self) -> float: ...
    def group_var(self, group_name: str, variable_name: str) -> float: ...
    def has_group_var(self, group_name: str, variable_name: str) -> bool: ...
    def has_well_var(self, well_name: str, variable_name: str) -> bool: ...
    def update(self, variable_name: str, value: float) -> None: ...
    def update_group_var(self, group_name: str, variable_name: str, new_value: float) -> None: ...
    def update_well_var(self, well_name: str, variable_name: str, new_value: float) -> None: ...
    def well_var(self, well_name: str, variable_name: str) -> float: ...
    def __contains__(self, variable_name: str) -> bool: ...
    def __getitem__(self, variable_name: str) -> float: ...
    def __setitem__(self, variable_name: str, new_value: float) -> None: ...
    @property
    def groups(self) -> List[str]: ...
    @property
    def wells(self) -> List[str]: ...

class Tables:
    def __init__(self, *args, **kwargs) -> None: ...
    def evaluate(self, arg0: str, arg1: int, arg2: str, arg3: float) -> float: ...
    def __contains__(self, arg0: str) -> bool: ...

class UDAValue:
    @overload
    def __init__(self, arg0: float, arg1) -> None: ...
    @overload
    def __init__(self, arg0: str, arg1) -> None: ...
    def dimension(self, *args, **kwargs) -> Any: ...
    def get_double(self) -> float: ...
    def get_string(self) -> str: ...
    def is_double(self) -> bool: ...
    def is_string(self) -> bool: ...

class UnitSystem:
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def name(self) -> str: ...

class Well:
    def __init__(self, *args, **kwargs) -> None: ...
    def available_gctrl(self) -> bool: ...
    def connections(self) -> List[Connection]: ...
    def group(self) -> str: ...
    def guide_rate(self) -> float: ...
    def isdefined(self, report_step: int) -> bool: ...
    def isinjector(self) -> bool: ...
    def isproducer(self) -> bool: ...
    def pos(self) -> Tuple[int,int,float]: ...
    def status(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def preferred_phase(self) -> str: ...

class action:
    __members__: ClassVar[dict] = ...  # read-only
    __entries: ClassVar[dict] = ...
    ignore: ClassVar[action] = ...
    throw: ClassVar[action] = ...
    warn: ClassVar[action] = ...
    def __init__(self, value: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __getstate__(self) -> int: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    def __setstate__(self, state: int) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class eclArrType:
    __members__: ClassVar[dict] = ...  # read-only
    C0nn: ClassVar[eclArrType] = ...
    CHAR: ClassVar[eclArrType] = ...
    DOUB: ClassVar[eclArrType] = ...
    INTE: ClassVar[eclArrType] = ...
    LOGI: ClassVar[eclArrType] = ...
    MESS: ClassVar[eclArrType] = ...
    REAL: ClassVar[eclArrType] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None: ...
    def __and__(self, other: object) -> object: ...
    def __eq__(self, other: object) -> bool: ...
    def __ge__(self, other: object) -> bool: ...
    def __getstate__(self) -> int: ...
    def __gt__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __invert__(self) -> object: ...
    def __le__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __or__(self, other: object) -> object: ...
    def __rand__(self, other: object) -> object: ...
    def __ror__(self, other: object) -> object: ...
    def __rxor__(self, other: object) -> object: ...
    def __setstate__(self, state: int) -> None: ...
    def __xor__(self, other: object) -> object: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class eclSectionType:
    __members__: ClassVar[dict] = ...  # read-only
    EDIT: ClassVar[eclSectionType] = ...
    GRID: ClassVar[eclSectionType] = ...
    PROPS: ClassVar[eclSectionType] = ...
    REGIONS: ClassVar[eclSectionType] = ...
    RUNSPEC: ClassVar[eclSectionType] = ...
    SCHEDULE: ClassVar[eclSectionType] = ...
    SOLUTION: ClassVar[eclSectionType] = ...
    SUMMARY: ClassVar[eclSectionType] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None: ...
    def __and__(self, other: object) -> object: ...
    def __eq__(self, other: object) -> bool: ...
    def __ge__(self, other: object) -> bool: ...
    def __getstate__(self) -> int: ...
    def __gt__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __invert__(self) -> object: ...
    def __le__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __or__(self, other: object) -> object: ...
    def __rand__(self, other: object) -> object: ...
    def __ror__(self, other: object) -> object: ...
    def __rxor__(self, other: object) -> object: ...
    def __setstate__(self, state: int) -> None: ...
    def __xor__(self, other: object) -> object: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

def calc_cell_vol(arg0: List[float[8]], arg1: List[float[8]], arg2: List[float[8]]) -> float: ...
def create_deck(arg0: str, arg1: ParseContext, arg2) -> Deck: ...
def create_deck_string(arg0: str, arg1: ParseContext, arg2) -> Deck: ...



import unittest
import sys
import numpy as np
import datetime

from opm.io.ecl import ERft, eclArrType

try:
    from tests.utils import test_path
except ImportError:
    from utils import test_path

class TestEclFile(unittest.TestCase):

    def test_content(self):

        refContent = [ ('PROD', (2015, 1, 1), 0.0),
                       ('INJ', (2015, 1, 1), 0.0),
                       ('A-1H', (2015, 9, 1), 243.0),
                       ('B-2H', (2016, 5, 31), 516.0),
                       ('PROD', (2017, 7, 31), 942.0) ]


        with self.assertRaises(RuntimeError):
            ERft("/file/that/does_not_exists")

        rft1 = ERft(test_path("data/SPE1CASE1.RFT"))
        self.assertEqual(len(rft1), 5)

        self.assertTrue( ("PROD", 2015, 1, 1) in rft1 )
        self.assertFalse( ("XXX", 2015, 1, 1) in rft1 )

        self.assertTrue( ("PRESSURE", "PROD", 2015, 1, 1) in rft1 )
        self.assertFalse( ("XXX", "PROD", 2015, 1, 1) in rft1 )

        self.assertTrue( ("PRESSURE", 0) in rft1 )
        self.assertTrue( ("PRESSURE", 1) in rft1 )

        self.assertFalse( ("XXX", 0) in rft1 )

        with self.assertRaises(ValueError):
            self.assertTrue( ("PRESSURE", "XX", "XX") in rft1 )

        rftlist = rft1.list_of_rfts

        self.assertEqual(len(rftlist), 5)

        for n, rftdata in enumerate(rftlist):
            self.assertEqual(rftdata[0], refContent[n][0])
            self.assertEqual(rftdata[1], refContent[n][1])
            self.assertEqual(rftdata[2], refContent[n][2])


    def test_list_of_rfts(self):

        ref_rftlist = []
        ref_rftlist.append(('PROD', (2015, 1, 1), 0.0))
        ref_rftlist.append(('INJ', (2015, 1, 1), 0.0))
        ref_rftlist.append(('A-1H', (2015, 9, 1), 243.0))
        ref_rftlist.append(('B-2H', (2016, 5, 31), 516.0))
        ref_rftlist.append(('PROD', (2017, 7, 31), 942.0))

        rft1 = ERft(test_path("data/SPE1CASE1.RFT"))

        for n, element in enumerate(rft1.list_of_rfts):
            self.assertEqual(element, ref_rftlist[n])


    def test_list_of_array(self):

        refArrList = ["TIME", "DATE", "WELLETC", "CONIPOS", "CONJPOS", "CONKPOS", "HOSTGRID", "DEPTH", "PRESSURE",
                      "SWAT", "SGAS"]

        rft1 = ERft(test_path("data/SPE1CASE1.RFT"))

        rft_date = (2016, 5, 31)
        arrList1 = rft1.arrays( "B-2H", rft_date )
        arrList2 = rft1.arrays( 3 )

        self.assertEqual(len(arrList1), len(refArrList))

        for n, (name, arrType, arrSize) in enumerate(arrList1):

            self.assertEqual(name, refArrList[n])

            if arrType != eclArrType.MESS:
                array = rft1[name, "B-2H", 2016, 5, 31]
                self.assertEqual(len(array), arrSize)

            if arrType == eclArrType.INTE:
                self.assertEqual(array.dtype, "int32")
            elif arrType == eclArrType.REAL:
                self.assertEqual(array.dtype, "float32")
            elif arrType == eclArrType.DOUB:
                self.assertEqual(array.dtype, "float64")
            elif arrType == eclArrType.LOGI:
                self.assertEqual(array.dtype, "bool")
            elif arrType == eclArrType.CHAR:
                self.assertTrue(array.dtype.kind in {'U', 'S'})


        self.assertEqual(len(arrList2), len(refArrList))

        for n, (name, arrType, arrSize) in enumerate(arrList2):

            self.assertEqual(name, refArrList[n])

            if arrType != eclArrType.MESS:
                array = rft1[name, "B-2H", 2016, 5, 31 ]
                self.assertEqual(len(array), arrSize)

            if arrType == eclArrType.INTE:
                self.assertEqual(array.dtype, "int32")
            elif arrType == eclArrType.REAL:
                self.assertEqual(array.dtype, "float32")
            elif arrType == eclArrType.DOUB:
                self.assertEqual(array.dtype, "float64")
            elif arrType == eclArrType.LOGI:
                self.assertEqual(array.dtype, "bool")
            elif arrType == eclArrType.CHAR:
                self.assertTrue(array.dtype.kind in {'U', 'S'})


    def test_get(self):

        pref = [0.57886104E+04, 0.57946934E+04, 0.58056177E+04]
        refI = [9, 9, 9]
        refJ = [4, 4, 4]
        refK = [1, 2, 3]

        rft1 = ERft(test_path("data/SPE1CASE1.RFT"))

        pres1a = rft1["PRESSURE", "A-1H", 2015, 9, 1 ]
        pres1b = rft1["PRESSURE", 2 ]

        self.assertEqual(len(pres1a), len(pref))
        self.assertEqual(len(pres1b), len(pref))

        for ref, v1, v2 in zip(pref, pres1a, pres1b):
            self.assertAlmostEqual(ref, v1, 4)
            self.assertAlmostEqual(ref, v2, 4)

        conI_1a = rft1["CONIPOS", "B-2H", 2016, 5, 31]
        conJ_1a = rft1["CONJPOS", "B-2H", 2016, 5, 31]
        conK_1a = rft1["CONKPOS", "B-2H", 2016, 5, 31]

        conI_1b = rft1["CONIPOS", 3]
        conJ_1b = rft1["CONJPOS", 3]
        conK_1b = rft1["CONKPOS", 3]

        self.assertEqual(len(refI), len(conI_1a))
        self.assertEqual(len(refI), len(conI_1b))

        self.assertEqual(len(refJ), len(conJ_1a))
        self.assertEqual(len(refJ), len(conJ_1b))

        self.assertEqual(len(refK), len(conK_1a))
        self.assertEqual(len(refK), len(conK_1b))

        for ref, v1, v2 in zip(refI, conI_1a, conI_1b):
            self.assertEqual(ref, v1)
            self.assertEqual(ref, v2)

        for ref, v1, v2 in zip(refJ, conJ_1a, conJ_1b):
            self.assertEqual(ref, v1)
            self.assertEqual(ref, v2)

        for ref, v1, v2 in zip(refK, conK_1a, conK_1b):
            self.assertEqual(ref, v1)
            self.assertEqual(ref, v2)


if __name__ == "__main__":

    unittest.main()



def run(ecl_state, schedule, report_step, sim, actionx_callback):
    wells_shut = False

    for well in sim.wells:
        if sim.well_var(well, "WWCT") > 0.50:
            schedule.shut_well(well, report_step)
            wells_shut = True

    return wells_shut



import datetime
from operator import attrgetter

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from opm.io.parser import Parser

# This is from the TimeMap.cpp implementation in opm
ecl_month = {
    "JAN": 1,
    "FEB": 2,
    "MAR": 3,
    "APR": 4,
    "MAI": 5,
    "MAY": 5,
    "JUN": 6,
    "JLY": 7,
    "JUL": 7,
    "AUG": 8,
    "SEP": 9,
    "OCT": 10,
    "OKT": 10,
    "NOV": 11,
    "DEC": 12,
    "DES": 12,
}

inv_ecl_month = {
    1: "JAN",
    2: "FEB",
    3: "MAR",
    4: "APR",
    5: "MAY",
    6: "JUN",
    7: "JUL",
    8: "AUG",
    9: "SEP",
    10: "OCT",
    11: "NOV",
    12: "DEC",
}


def _make_datetime(dates_record):
    day = dates_record[0].get_int(0)
    month = dates_record[1].get_str(0)
    year = dates_record[2].get_int(0)

    date_dt = datetime.datetime(year, ecl_month[month], day)
    if len(dates_record) < 4:
        return date_dt
    else:
        time_str = dates_record[3].get_str(0)
        time_list = time_str.split(":")
        hour = minute = second = microsecond = 0
        hour = int(time_list[0])
        if len(time_list) > 1:
            minute = int(time_list[1])
        if len(time_list) > 2:
            sec_list = time_list[2].split(".")
            second = int(sec_list[0])
            if len(sec_list) > 1:
                ms_str = sec_list[1].strip()
                npad = 6 - len(ms_str)
                ms_str += "".join(["0" for i in range(npad)])
                microsecond = int(ms_str)

        return datetime.datetime(
            year, ecl_month[month], day, hour, minute, second, microsecond
        )


class TimeStep(object):
    def __init__(self, dt, keywords):
        """The TimeStep class consist of a list of keywords and a corresponding date.

        Observe that the date value corresponds to a DATES / TSTEP keyword
        following *after* the keywords; i.e. if the TimeStep instance contains
        a WCONHIST keyword the settings in that keyword should apply *until*
        the date specified is reached. See the documentation of the TimeVector
        class for more details of the relationship between TimeVector and
        TimeStep.

        """
        self.dt = dt
        self.keywords = keywords
        self.tstep = None
        self.is_start = False

    @classmethod
    def create_first(cls, dt):
        ts = cls(dt, [])
        ts.is_start = True
        return ts

    def add_keyword(self, kw):
        self.keywords.append(kw)

    def __len__(self):
        return len(self.keywords)

    def __contains__(self, arg):
        for kw in self.keywords:
            if arg == kw.name:
                return True
        return False

    def __str__(self):
        string = StringIO()

        if not self.is_start:
            day = self.dt.day
            month = self.dt.month
            year = self.dt.year
            if not self.dt.time():
                string.write(
                    "DATES\n  {day} '{month}' {year}/\n/\n\n".format(
                        day=day, month=inv_ecl_month[month], year=year
                    )
                )
            else:
                hour = self.dt.hour
                minute = self.dt.minute
                second = self.dt.second + self.dt.microsecond * 1.0e-6
                if self.dt.microsecond > 0:
                    string.write(
                        "DATES\n  {day} '{month}' {year}"
                        " {hour:02d}:{minute:02d}:{second:07.4f} /\n/\n\n".format(
                            day=day,
                            month=inv_ecl_month[month],
                            year=year,
                            hour=hour,
                            minute=minute,
                            second=second,
                        )
                    )
                else:
                    string.write(
                        "DATES\n  {day} '{month}' {year}"
                        " {hour:02d}:{minute:02d}:{second:02d} /\n/\n\n".format(
                            day=day,
                            month=inv_ecl_month[month],
                            year=year,
                            hour=hour,
                            minute=minute,
                            second=int(second),
                        )
                    )

        for kw in self.keywords:
            string.write(str(kw))
            string.write("\n")

        return string.getvalue()


class TimeVector(object):
    def __init__(self, start_date, base_string=None, base_file=None):
        """The TimeVector class is a simple vector class with DATES/TSTEP blocks.

         The TimeVector class is a basic building block for tools designed to
         update schedule files. A schedule file consists of a list of keywords
         related to the dynamic properties of the field, like opening and
         closing wells, specifiying rates and so on. The temporal advancement of
         the simulator is controlled by DATES and TSTEP keywords. A typical
         schedule section can look like this:

         --- Step 1 -----------------------

         WELSPECS
            'C1'   'G1'   10 10 10 'OIL' /
         /

         COMPDAT
            'C1' 15 20 10 16 'OPEN' /
            'C1' 15 21 16 16 'OPEN' /
         /

         WCONHIST
            'C1' 'OPEN'  'ORAT' 1000 /
         /

         --- Step 2 ----------------------

         DATES
            10  'MAY' 2016 /
         /

         WCONHIST
            'C1'  'OPEN'  'ORAT' 2000 /
         /

        --- Step 3 ----------------------

         TSTEP
             10 /

         WELSPECS
            'W2'  'G1'  5 5 5 'OIL' /
         /

         COMPDAT
            'W2' 10 10 7 10 'OPEN' /
         /

         WCONHIST
            'C1' 'OPEN' 'ORAT' 3000 /
            'W2' 'OPEN' 'ORAT' 1500 /
         /

         --- Step 4 ----------------------

         DATES
            30 'MAY' 2016 /
         /

         As indicated above the DATES and TSTEP keywords act as delimiters in
         the schedule file. In the TimeVector class the fundamental unit is
         TimeStep instance which consists of a list of keywords, and a
         terminating DATES or TSTEP keyword, the example above would correspond
         to a TimeVector with three TimeStep instances.

         Basic usage example:

            #!/usr/bin/env python
            from opm.tools import TimeVector

            # Create vector and load history.
            tv = TimeVector( start )
            tv.load("history.sch")


            # Load predictions from another file
            tv.load("prediction.sch")


            # Insert the definition of one particular well at
            # a specifed date.
            tv.load("extra_wll.sch", date = datetime.datetime(2018,10,1))


            # Check if we have a certain timestep:
            if datetime.datetime(2017,1,1) in tv:
                print("We have it!")
            else:
                print("No such date")


            # Dump the updated schedule content to a file:
            with open("schedule","w") as f:
                 f.write(str(tv))


        """
        if base_string and base_file:
            raise ValueError(
                "Can only supply one of base_string and base_file arguments"
            )

        self.start_date = datetime.datetime(
            start_date.year, start_date.month, start_date.day
        )
        self.time_steps_dict = {}
        self.time_steps_list = []

        ts = TimeStep.create_first(self.start_date)

        self._add_dates_block(ts)
        start_dt = datetime.datetime(start_date.year, start_date.month, start_date.day)
        if base_file:
            deck = Parser().parse(base_file)
            self._add_deck(deck, start_dt)

        if base_string:
            deck = Parser().parse_string(base_string)
            self._add_deck(deck, start_dt)

    def __len__(self):
        """
        The number of timesteps in the vector.
        """
        return len(self.time_steps_dict)

    def __contains__(self, dt):
        """
        Will return true if the vector contains a timestep at date dt.
        """
        if isinstance(dt, datetime.date):
            dt = datetime.datetime(dt.year, dt.month, dt.day)
        return dt in self.time_steps_dict

    def __getitem__(self, index):
        """Will look up a timestep in the vector.

        The index argument can either be an integer or a datetime.date/datetime
        instance.

        """
        if isinstance(index, int):
            return self.time_steps_list[index]
        else:
            if not isinstance(index, datetime.datetime) and isinstance(
                index, datetime.date
            ):
                index = datetime.datetime(index.year, index.month, index.day)

            return self.time_steps_dict[index]

    def _add_dates_block(self, ts):
        self.time_steps_dict[ts.dt] = ts
        self.time_steps_list.append(ts)

    def delete(self, dt):
        del self.time_steps_dict[dt]
        for (index, ts) in enumerate(self.time_steps_list):
            if ts.dt == dt:
                del self.time_steps_list[index]
                break

    def add_keywords(self, dt, keywords):
        if dt < self.start_date:
            raise ValueError("Invalid datetime argument: {}".format(dt))

        if dt in self.time_steps_dict:
            ts = self[dt]
            for kw in keywords:
                ts.add_keyword(kw)
        else:
            ts = TimeStep(dt, keywords)
            self._add_dates_block(ts)
            self.time_steps_list.sort(key=attrgetter("dt"))

    def _add_deck(self, deck, start_date):
        first_kw = deck[0]
        if start_date is None:
            if first_kw.name != "DATES":
                raise ValueError(
                    "When loading you must *either* specify date - or file must start"
                    " with DATES keyword"
                )
            dt = _make_datetime(first_kw[len(first_kw) - 1])
        else:
            if first_kw.name == "DATES":
                raise ValueError(
                    "When loading you must *either* specify date - or file must start"
                    " with DATES keyword"
                )
            dt = start_date

        keywords = []
        for kw in deck:

            if kw.name == "DATES":
                self.add_keywords(dt, keywords)

                for index in range(len(kw) - 1):
                    dt = _make_datetime(kw[index])
                    self.add_keywords(dt, [])

                dt = _make_datetime(kw[len(kw) - 1])

                keywords = []
                continue

            # if kw.name == "TSTEP":
            # raise ValueError("Must block the ranges with active TSTEP -
            #         getting a DATES in there is ERROR")

            keywords.append(kw)

        self.add_keywords(dt, keywords)

    def load(self, filename, date=None):
        """Will parse a Schedule file and add the keywords to the current TimeVector.

        You can call the load() method repeatedly, the different timesteps will
        be ordered chronologically. If a timestep is already present the
        keywords will be appended.

        The optional date argument can be used to insert schedule file
        fragments which do not have any DATES / TSTEP keywords. Assuming you
        have a base file 'base.sch' and a small fragment 'well.sch' with the
        WELSPECS and COMPDAT keywords to create one well, then the new well can
        be added 1.st of April 2017 as this:

            tv = TimeVector( start )
            tv.load("base.sch")
            tv.load("well.sch", date = datetime.datetime(2017, 4, 1))

        """
        deck = Parser().parse(filename)
        self._add_deck(deck, date)

    def load_string(self, deck_string, date=None):
        """
        Like load() - but load from a string literal instead of file.
        """
        deck = Parser().parse_string(deck_string)
        self._add_deck(deck, date)

    def __str__(self):
        """Will return a string representation of the vector.

        The output from this method should be valid Schedule input which can be
        passed to a simulator.

        """

        string = StringIO()
        for ts in self:
            string.write(str(ts))

        return string.getvalue()

    @property
    def dates(self):
        """
        Will return a list of all the dates in the vector.
        """
        return [x.dt for x in self.time_steps_list]



import unittest

from opm.io.parser import Parser

class TestParse(unittest.TestCase):

    DECK_STRING = """
START             -- 0
10 MAI 2007 /
RUNSPEC

DIMENS
2 2 1 /
GRID
DX
4*0.25 /
DY
4*0.25 /
DZ
4*0.25 /
TOPS
4*0.25 /
REGIONS
OPERNUM
3 3 1 2 /
FIPNUM
1 1 2 3 /
"""

    def setUp(self):
        self.deck = Parser().parse_string(self.DECK_STRING)

    def test_deck_in(self):
        map(lambda kw: self.assertIn(kw, self.deck), [
            'START', 'RUNSPEC', 'DIMENS',
            'GRID', 'TOPS', 'REGIONS',
            'OPERNUM', 'FIPNUM'
        ])
        map(lambda kw: self.assertNotIn(kw, self.deck), [
            'PPCWMAX', 'APIGROUP', 'NOWARN', 'WARN', 'TBLKFAI4', 'WINJMULT'
        ])

    def test_deck_str(self):
        self.assertEqual(
            'DIMENS 2 2 1 /'.split(),
            str(self.deck['DIMENS']).split()
        )
        self.assertEqual(
            'DX 0.25 0.25 0.25 0.25 /'.split(),
            str(self.deck['DX']).split()
        )
        self.assertEqual(
            str(  Parser().parse_string('RUNSPEC\n\nDX\n4*0.5 /')  ).split(),
            'RUNSPEC DX 0.5 0.5 0.5 0.5 /'.split()
        )

    def test_deck_keyword(self):
        for kw in self.deck:
            self.assertTrue(len(kw.name) > 0)
            for rec in kw:
                self.assertTrue( len(rec) > 0 )
                for item in rec:
                    self.assertTrue(len(item) > 0)

    def test_deck_FIPNUM(self):
        self.assertIn('FIPNUM', self.deck)
        self.assertEqual(len(self.deck['FIPNUM']), 1)
        self.assertEqual(len(self.deck['FIPNUM'][0]), 1)
        self.assertEqual(len(self.deck['FIPNUM'][0][0].get_data_list()), 4)


if __name__ == "__main__":
    unittest.main()



import unittest

from opm.io.parser import Parser
from opm.io.ecl_state import EclipseState
from opm.io.schedule import Schedule
try:
    from tests.utils import test_path
except ImportError:
    from utils import test_path

def flowing(connection):
    return connection.state == 'OPEN'

def closed(connection):
    return connection.state == 'SHUT'


class TestWells(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        deck = Parser().parse(test_path('spe3/SPE3CASE1.DATA'))
        print("Creating state")
        cls.state = EclipseState(deck)
        print("State OK")
        cls.sch = Schedule(deck, cls.state)
        cls.reportsteps = cls.sch.reportsteps

    def test_connection_pos(self):
        wells = self.sch.get_wells(0)
        p00 = wells[0].connections()[0].pos
        p01 = wells[0].connections()[1].pos
        p10 = wells[1].connections()[0].pos
        p11 = wells[1].connections()[1].pos
        self.assertEqual(p00, (6,6,2))
        self.assertEqual(p01, (6,6,3))
        self.assertEqual(p10, (0,0,0))
        self.assertEqual(p11, (0,0,1))

    def test_connection_state(self):
        for reportstep,_ in enumerate(self.reportsteps):
            for well in self.sch.get_wells(reportstep):
                for connection in well.connections():
                    self.assertEqual("OPEN", connection.state)

    def test_filters(self):
        connections = self.sch.get_wells(0)[0].connections()
        self.assertEqual(len(list(filter(flowing, connections))), 2)
        self.assertEqual(len(list(filter(closed, connections))), 0)

    def test_direction(self):
       for reportstep,_ in enumerate(self.reportsteps):
           for well in self.sch.get_wells(reportstep):
                for connection in well.connections():
                    self.assertEqual(connection.direction, 'Z')

    def test_attached_to_segment(self):
        for reportstep,_ in enumerate(self.reportsteps):
            for well in self.sch.get_wells(reportstep):
                for connection in well.connections():
                    self.assertFalse(connection.attached_to_segment)


if __name__ == "__main__":
    unittest.main()



import sys

def run(ecl_state, schedule, report_step, summary_state, actionx_callback):
    if (report_step == 1):
        schedule.shut_well("PROD2", report_step)
    sys.stdout.write("Running PYACTION arg1:{}\n".format(ecl_state))
    if "FOPR" in summary_state:
        sys.stdout.write("Have FOPR: {}\n".format( summary_state["FOPR"] ))
    else:
        sys.stdout.write("Missing FOPR\n")

    grid = ecl_state.grid()
    sys.stdout.write("Grid dimensions: ({},{},{})\n".format(grid.nx, grid.ny, grid.nz))

    prod_well = schedule.get_well("PROD1", report_step)
    sys.stdout.write("Well status: {}\n".format(prod_well.status()))
    if not "list" in storage:
        storage["list"] = []
    storage["list"].append(report_step)
    sys.stdout.write("storage[list]: {}\n".format(storage["list"]))

    if summary_state.well_var("PROD1", "WWCT") > 0.80:
        schedule.shut_well("PROD1", report_step)
        schedule.open_well("PROD2", report_step)
        summary_state.update("RUN_COUNT", 1)
    return True



import unittest
import opm.io

import numpy as np

from opm.io.parser import Parser
from opm.io.deck import DeckKeyword
from opm.io.ecl_state import EclipseState
try:
    from tests.utils import test_path
except ImportError:
    from utils import test_path


class TestFieldProps(unittest.TestCase):

    def assertClose(self, expected, observed, epsilon=1e-08):
        diff = abs(expected - observed)
        err_msg = '|%g - %g| = %g > %g' % (expected, observed, diff, epsilon)
        self.assertTrue(diff <= epsilon, msg=err_msg)

    def setUp(self):
        parser = Parser()
        deck = parser.parse(test_path('spe3/SPE3CASE1.DATA'))
        int_array = np.ones(324)
        actnum_kw = DeckKeyword( parser["ACTNUM"], int_array)
        deck.add(actnum_kw)
        self.spe3 = EclipseState(deck)
        self.props = self.spe3.field_props()

    def test_contains(self):
        p = self.props
        self.assertTrue('PORO' in p)
        self.assertFalse('NONO' in p)
        self.assertTrue('PORV' in p)
        self.assertTrue('ACTNUM' in p)


    def test_getitem(self):
        p = self.props
        poro = p.get_double_array('PORO')
        self.assertEqual(324, len(poro))
        self.assertEqual(0.13, poro[0])
        self.assertTrue( 'PERMX' in p )
        px = p.get_double_array('PERMX')
        print(len(px))
        self.assertEqual(324, len(px))
        self.assertEqual(324, len(p.get_int_array('ACTNUM')))

    def test_permx_values(self):
        def md2si(md):
            #millidarcy->SI
            return md * 1e-3 * 9.869233e-13
        field_props = self.props

        grid  = self.spe3.grid()
        permx = field_props.get_double_array('PERMX')
        print('set(PERMX) = %s' % set(permx))
        # 130mD, 40mD, 20mD, and 150mD, respectively, top to bottom
        darcys = {0:md2si(130), 1:md2si(40), 2:md2si(20), 3:md2si(150)}
        for i in range(grid.nx):
            for j in range(grid.ny):
                for k in range(grid.nz):
                    g_idx = grid.globalIndex(i,j,k)
                    perm  = permx[g_idx]
                    darcy = darcys[k]
                    self.assertClose(darcy, perm)

    def test_volume(self):
        grid  = self.spe3.grid()
        for i in range(grid.nx):
            for j in range(grid.ny):
                for k in range(grid.nz):
                    g_idx = grid.globalIndex(i,j,k)
                    exp = 293.3 * 293.3 * 30  # cubicfeet = 73 078.6084 cubic meter
                    exp *= (12*0.0254)**3  # cubic feet to cubic meter
                    if k == 0:
                        self.assertClose(exp, grid.getCellVolume(g_idx))
                    self.assertEqual(grid.getCellVolume(g_idx), grid.getCellVolume(i, j, k))



import unittest
import sys
import numpy as np
import datetime

from opm.io.ecl import ESmry
try:
    from tests.utils import test_path
except ImportError:
    from utils import test_path


class TestEclFile(unittest.TestCase):

    def test_base_runs(self):

        with self.assertRaises(ValueError):
            ESmry("/file/that/does_not_exists")

        smry1 = ESmry(test_path("data/SPE1CASE1.SMSPEC"))
        smry1.make_esmry_file()

        smry2 = ESmry(test_path("data/SPE1CASE1.ESMRY"))

        with self.assertRaises(ValueError):
            smry2.make_esmry_file()

        self.assertEqual(len(smry1), 67)
        self.assertEqual(smry1.start_date, datetime.datetime(2015,1,1) )
        self.assertEqual(smry1.end_date, datetime.datetime(2020,4,29) )

        self.assertTrue("TIME" in smry1)
        self.assertTrue("BPR:10,10,3" in smry1)
        self.assertFalse("XXXX" in smry1)

        dates = smry1.dates()
        self.assertEqual(len(dates), len(smry1))
        self.assertEqual(dates[0], smry1.start_date + datetime.timedelta(days=1))

        self.assertEqual(smry1.units('FGOR'), 'STB/MSCF')

        with self.assertRaises(ValueError):
            test = smry1["XXX"]

        time1a = smry1["TIME"]

        self.assertEqual(len(time1a), len(smry1))

        time1b = smry1["TIME", True]

        self.assertEqual(len(time1b), 64)

    def test_start_date(self):

        smry1 = ESmry(test_path("data/T1_STARTD.SMSPEC"))
        smry2 = ESmry(test_path("data/T1_STARTD.ESMRY"))

        self.assertEqual(smry1.start_date, datetime.datetime(2020,3,29, 5, 4, 41) )
        self.assertEqual(smry2.start_date, datetime.datetime(2020,3,29, 5, 4, 41) )

        self.assertEqual(smry1.end_date, datetime.datetime(2020,4,24, 5, 4, 41) )
        self.assertEqual(smry2.end_date, datetime.datetime(2020,4,24, 5, 4, 41) )


    def test_restart_runs(self):

        base_smry = ESmry(test_path("data/SPE1CASE1.SMSPEC"))
        self.assertEqual(base_smry.start_date, datetime.datetime(2015,1,1) )
        time0 = base_smry["TIME"]
        self.assertEqual(time0[0], 1.0)

        rst_smry1 = ESmry(test_path("data/SPE1CASE1_RST60.SMSPEC"))
        time1 = rst_smry1["TIME"]
        gor1 = rst_smry1["WGOR:PROD"]

        self.assertEqual(rst_smry1.start_date, datetime.datetime(2015,1,1) )
        self.assertEqual(len(rst_smry1), 60)
        self.assertEqual(time1[0], 1856.0)
        self.assertEqual(len(time1), 60)
        self.assertEqual(len(gor1), 60)

        rst_smry2 = ESmry(test_path("data/SPE1CASE1_RST60.SMSPEC"), load_base_run = True)
        time2 = rst_smry2["TIME"]
        gor2 = rst_smry2["WGOR:PROD"]

        self.assertEqual(rst_smry2.start_date, datetime.datetime(2015,1,1) )
        self.assertEqual(len(rst_smry2), 123)

        self.assertEqual(time2[0], 1.0)
        self.assertEqual(len(rst_smry2), 123)
        self.assertEqual(len(time2), 123)
        self.assertEqual(len(gor2), 123)


    def test_keylist(self):

        ref_key_list = ["BPR:1,1,1", "BPR:10,10,3", "FGOR", "FOPR", "TIME", "WBHP:INJ", "WBHP:PROD",
                        "WGIR:INJ", "WGIR:PROD", "WGIT:INJ", "WGIT:PROD", "WGOR:PROD", "WGPR:INJ",
                        "WGPR:PROD", "WGPT:INJ", "WGPT:PROD", "WOIR:INJ", "WOIR:PROD", "WOIT:INJ",
                        "WOIT:PROD", "WOPR:INJ", "WOPR:PROD", "WOPT:INJ", "WOPT:PROD", "WWIR:INJ",
                        "WWIR:PROD", "WWIT:INJ", "WWIT:PROD", "WWPR:INJ", "WWPR:PROD", "WWPT:INJ",
                        "WWPT:PROD"]

        ref_keys_pattern = ["WGPR:INJ", "WGPR:PROD", "WOPR:INJ", "WOPR:PROD", "WWPR:INJ", "WWPR:PROD"]

        smry1 = ESmry(test_path("data/SPE1CASE1.SMSPEC"))

        list_of_keys = smry1.keys()
        self.assertEqual(len(list_of_keys), len(ref_key_list))

        for key, ref_key in zip(list_of_keys, ref_key_list):
            self.assertEqual(key, ref_key)

        for key in list_of_keys:
            data = smry1[key]
            self.assertEqual(len(smry1), len(data))

        list_of_keys2 = smry1.keys("W?PR:*")

        self.assertEqual(len(list_of_keys2), len(ref_keys_pattern))

        for key, ref in zip(list_of_keys2, ref_keys_pattern):
            self.assertEqual(key, ref)


    def test_base_runs_ext(self):

        smry1 = ESmry(test_path("data/SPE1CASE1.SMSPEC"))
        smry1.make_esmry_file()

        extsmry1 = ESmry(test_path("data/SPE1CASE1.ESMRY"))

        self.assertEqual(len(extsmry1), 67)
        self.assertEqual(extsmry1.start_date, datetime.datetime(2015,1,1) )
        self.assertEqual(extsmry1.end_date, datetime.datetime(2020,4,29) )

        self.assertTrue("TIME" in extsmry1)
        self.assertTrue("BPR:10,10,3" in extsmry1)
        self.assertFalse("XXXX" in extsmry1)

        with self.assertRaises(ValueError):
            test = extsmry1["XXX"]

        time1a = extsmry1["TIME"]

        self.assertEqual(len(time1a), len(extsmry1))

        time1b = extsmry1["TIME", True]

        self.assertEqual(len(time1b), 64)


    def test_restart_runs_ext(self):

        base_smry = ESmry(test_path("data/SPE1CASE1.SMSPEC"))
        base_smry.make_esmry_file()

        base_ext_smry = ESmry(test_path("data/SPE1CASE1.ESMRY"))

        time0 = base_ext_smry["TIME"]
        self.assertEqual(time0[0], 1.0)

        rst_smry1 = ESmry(test_path("data/SPE1CASE1_RST60.SMSPEC"))

        rst_ext_smry1 = ESmry(test_path("data/SPE1CASE1_RST60.ESMRY"))

        time1 = rst_ext_smry1["TIME"]
        gor1 = rst_ext_smry1["WGOR:PROD"]

        self.assertEqual(base_ext_smry.start_date, datetime.datetime(2015,1,1) )
        time0 = base_ext_smry["TIME"]
        self.assertEqual(time0[0], 1.0)

        rst_ext_smry1 = ESmry(test_path("data/SPE1CASE1_RST60.ESMRY"))
        time1 = rst_ext_smry1["TIME"]
        gor1 = rst_ext_smry1["WGOR:PROD"]

        self.assertEqual(rst_smry1.start_date, datetime.datetime(2015,1,1) )
        self.assertEqual(len(rst_smry1), 60)
        self.assertEqual(time1[0], 1856.0)
        self.assertEqual(len(time1), 60)
        self.assertEqual(len(gor1), 60)

        rst_ext_smry2 = ESmry(test_path("data/SPE1CASE1_RST60.ESMRY"), load_base_run = True)
        time2 = rst_ext_smry2["TIME"]
        gor2 = rst_ext_smry2["WGOR:PROD"]

        self.assertEqual(rst_ext_smry2.start_date, datetime.datetime(2015,1,1) )
        self.assertEqual(len(rst_ext_smry2), 123)

        self.assertEqual(time2[0], 1.0)
        self.assertEqual(len(rst_ext_smry2), 123)
        self.assertEqual(len(time2), 123)
        self.assertEqual(len(gor2), 123)


    def test_keylist_ext(self):

        ref_key_list = ["BPR:1,1,1", "BPR:10,10,3", "FGOR", "FOPR", "TIME", "WBHP:INJ", "WBHP:PROD",
                        "WGIR:INJ", "WGIR:PROD", "WGIT:INJ", "WGIT:PROD", "WGOR:PROD", "WGPR:INJ",
                        "WGPR:PROD", "WGPT:INJ", "WGPT:PROD", "WOIR:INJ", "WOIR:PROD", "WOIT:INJ",
                        "WOIT:PROD", "WOPR:INJ", "WOPR:PROD", "WOPT:INJ", "WOPT:PROD", "WWIR:INJ",
                        "WWIR:PROD", "WWIT:INJ", "WWIT:PROD", "WWPR:INJ", "WWPR:PROD", "WWPT:INJ",
                        "WWPT:PROD"]

        ref_keys_pattern = ["WGPR:INJ", "WGPR:PROD", "WOPR:INJ", "WOPR:PROD", "WWPR:INJ", "WWPR:PROD"]

        smry1 = ESmry(test_path("data/SPE1CASE1.SMSPEC"))
        smry1.make_esmry_file()

        ext_smry1 = ESmry(test_path("data/SPE1CASE1.ESMRY"))

        list_of_keys = ext_smry1.keys()
        self.assertEqual(len(list_of_keys), len(ref_key_list))

        for key, ref_key in zip(list_of_keys, ref_key_list):
            self.assertEqual(key, ref_key)

        for key in list_of_keys:
            data = smry1[key]
            self.assertEqual(len(smry1), len(data))

        list_of_keys2 = ext_smry1.keys("W?PR:*")

        self.assertEqual(len(list_of_keys2), len(ref_keys_pattern))

        for key, ref in zip(list_of_keys2, ref_keys_pattern):
            self.assertEqual(key, ref)



if __name__ == "__main__":

    unittest.main()



from opm.io.parser import Parser
from opm.io.ecl_state import EclipseState
from opm.io.schedule import Schedule

def main():
    deck = Parser().parse('../tests/spe3/SPE3CASE1.DATA')
    es = EclipseState(deck)
    sc = Schedule(deck, es)
    wp = sc.get_wells(0)[0] # producer
    wi = sc.get_wells(0)[1] # injector
    print('state:     %s' % es)
    print('schedule:  %s' % sc)
    print('prod well: %s' % wp)
    print('inj  well: %s' % wi)
    for i in range(len(sc.reportsteps)):
        if not sc.get_wells(i)[0].isproducer() or sc.get_wells(i)[0].isinjector():
            print('wp is not producer in step %s' % sc.reportsteps[i])
        if not sc.get_wells(i)[1].isinjector() or sc.get_wells(i)[1].isproducer():
            print('wi is not injector in step %s' % sc.reportsteps[i])

if __name__ == '__main__':
    main()



import opm_embedded

schedule = opm_embedded.current_schedule
report_step = opm_embedded.current_report_step

if (report_step == 2):
	invalid_kw = """
	GRID

	PORO
	    1000*0.1 /
	PERMX
	    1000*1 /
	PERMY
	    1000*0.1 /
	PERMZ
	    1000*0.01 /
	"""
	schedule.insert_keywords(invalid_kw)



def run(ecl_state, schedule, report_step, summary_state, actionx_callback):

    if report_step == 1:
        schedule.shut_well("P1")
    elif report_step == 2:
        schedule.shut_well("P2")
    elif report_step == 3:
        schedule.shut_well("P3")
    elif report_step == 4:
        schedule.shut_well("P4")
    elif report_step == 5:
        schedule.open_well("P1")
